###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Widget layout and setup
$Id: widget.py 4275 2015-07-25 13:28:27Z roger.ineichen $
"""
__docformat__ = 'restructuredtext'

import zope.i18n
import zope.interface
import zope.component
from zope.schema.fieldproperty import FieldProperty

import z3c.form.field
import z3c.form.widget
import z3c.form.interfaces

from j01.form import interfaces


class WidgetMixin(object):
    """Enhanced widget layout mixin class supporting widget addons"""

    _showLabel = None
    _showRequired = None
    _showError = None
    _showDescription = None

    # description and addon set by updateWidget in j01 forms
    description = FieldProperty(interfaces.IWidget['description'])
    addOnBefore = FieldProperty(interfaces.IWidget['addOnBefore'])
    addOnAfter = FieldProperty(interfaces.IWidget['addOnAfter'])
    addOnWrapper = FieldProperty(interfaces.IWidget['addOnWrapper'])

    @apply
    def showLabel():
        """Show label condition

        Allows to disable the label for single (sequence) checkbox widgets
        override this in your widget if needed.
        """
        def fget(self):
            if self._showLabel is not None:
                # return overriden value
                return self._showLabel
            else:
                # depends on label
                return self.label is not None
        def fset(self, value):
            self._showLabel = value
        return property(fget, fset)

    @apply
    def showRequired():
        def fget(self):
            if self._showRequired is not None:
                # return overriden value
                return self._showRequired
            else:
                # depends on required marker
                return self.required is True
        def fset(self, value):
            self._showRequired = value
        return property(fget, fset)

    @apply
    def showError():
        def fget(self):
            if self._showError is not None:
                # return overriden value
                return self._showError
            else:
                # depends on error
                return self.error is not None
        def fset(self, value):
            self._showError = value
        return property(fget, fset)

    @apply
    def showDescription():
        def fget(self):
            if self._showDescription is not None:
                # return overriden value
                return self._showDescription
            else:
                # depends on description
                return self.description is not None
        def fset(self, value):
            self._showDescription = value
        return property(fget, fset)

    def render(self):
        """Render the plain widget without additional layout"""
        widget = super(WidgetMixin, self).render()
        classes = ['input-group']
        if self.addOnBefore is not None:
            widget = '%s\n%s' % (self.addOnBefore, widget)
            classes.append('input-group-before')
        if self.addOnAfter is not None:
            widget = '%s\n%s' % (widget, self.addOnAfter)
            classes.append('input-group-after')
        if self.addOnWrapper:
            # don't use if None or an empty string
            widget = self.addOnWrapper % {'widget': widget,
                                          'class': ' '.join(classes)}
        return widget



##############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: datepicker.py 4754 2018-02-15 04:53:32Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface

import z3c.form.widget

import j01.datepicker.widget

from j01.form import interfaces
from j01.form.widget.widget import WidgetMixin


# date widget
class DatePickerWidget(WidgetMixin, j01.datepicker.widget.DatePickerWidget):
    """DatePicker widget implementation."""

    zope.interface.implementsOnly(interfaces.IDatePickerWidget)

    klass = u'datepicker-control form-control'
    # prefix datepicker class and use bootstrap-datepicker. This prevents to
    # messup with the div.datepicker box
    css = u'bootstrap-datepicker'


def getDatePickerWidget(field, request):
    """IFieldWidget factory for ItemsWidget."""
    return z3c.form.widget.FieldWidget(field, DatePickerWidget(request))


def getStartDatePickerWidget(field, request):
    """IFieldWidget factory for ItemsWidget."""
    # this widget uses the default timeAppendix settings '00:00:00'
    # and prevents selecting previous dates
    widget = DatePickerWidget(request)
    widget.skipPastDates = True
    return z3c.form.widget.FieldWidget(field, widget)


def getEndDatePickerWidget(field, request):
    """IFieldWidget factory for ItemsWidget."""
    # this widget uses the default timeAppendix settings '23:59:59'
    # and prevents selecting previous dates
    widget = DatePickerWidget(request)
    widget.timeAppendix = '23:59:59'
    widget.skipPastDates = True
    return z3c.form.widget.FieldWidget(field, widget)



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Tests
$Id: tests.py 4627 2017-07-10 02:26:31Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import unittest
import doctest

import z3c.form.testing
import j01.form.testing


def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite('../README.txt'),
        doctest.DocFileSuite('checker.txt'),
        # widgets
        doctest.DocFileSuite('dictionary.txt',
            setUp=j01.form.testing.setUp, tearDown=j01.form.testing.tearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            ),
        doctest.DocFileSuite('password.txt',
            setUp=j01.form.testing.setUp, tearDown=j01.form.testing.tearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            ),
        doctest.DocFileSuite('field.txt',
            setUp=j01.form.testing.setUp, tearDown=j01.form.testing.tearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            checker=z3c.form.testing.outputChecker,
            ),
        ))


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Testing

$Id: testing.py 4241 2015-05-18 09:38:13Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import os.path

import zope.schema.interfaces
import zope.pagetemplate.interfaces

import z3c.form.interfaces
import z3c.form.testing
import z3c.form.widget

import p01.schema.interfaces

import j01.form.layer
import j01.form.interfaces
import j01.form.widget
import j01.form.widget.dictionary
import j01.form.widget.multi
import j01.form.widget.password
import j01.form.widget.radio
import j01.form.widget.select
import j01.form.widget.text
import j01.form.widget.textarea


def getPath(fName):
    return os.path.join(os.path.dirname(j01.form.widget.__file__), fName)


def provideWidgetTemplate(fName, iface, mode=z3c.form.interfaces.INPUT_MODE):
    fPath = getPath(fName)
    template = z3c.form.widget.WidgetTemplateFactory(fPath, 'text/html')
    zope.interface.directlyProvides(template,
        zope.pagetemplate.interfaces.IPageTemplate)
    zope.component.provideAdapter(template,
        (None, j01.form.layer.IFormLayer, None, None, iface),
        zope.pagetemplate.interfaces.IPageTemplate, name=mode)


def setupFormDefaults():
    # z3c.form widget setup
    z3c.form.testing.setupFormDefaults()

    # checkbox
    provideWidgetTemplate('checkbox_input.pt',
        j01.form.interfaces.ICheckBoxWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('checkbox_display.pt',
        j01.form.interfaces.ICheckBoxWidget,
        z3c.form.interfaces.DISPLAY_MODE)
    provideWidgetTemplate('checkbox_hidden.pt',
        j01.form.interfaces.ICheckBoxWidget,
        z3c.form.interfaces.HIDDEN_MODE)

    provideWidgetTemplate('checkbox_inline_input.pt',
        j01.form.interfaces.ICheckBoxInlineWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('checkbox_input.pt',
        j01.form.interfaces.ISingleCheckBoxWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('checkbox_inline_input.pt',
        j01.form.interfaces.ISingleCheckBoxInlineWidget,
        z3c.form.interfaces.INPUT_MODE)

    provideWidgetTemplate('checkbox_picker_input.pt',
        j01.form.interfaces.ICheckBoxPickerWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('checkbox_inline_picker_input.pt',
        j01.form.interfaces.ICheckBoxInlinePickerWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('checkbox_picker_input.pt',
        j01.form.interfaces.ISingleCheckBoxPickerWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('checkbox_inline_picker_input.pt',
        j01.form.interfaces.ISingleCheckBoxInlinePickerWidget,
        z3c.form.interfaces.INPUT_MODE)
    # dictionary
    zope.component.provideAdapter(
        j01.form.widget.dictionary.DictKeyValueConverter)

    # file
    provideWidgetTemplate('file_input.pt',
        j01.form.interfaces.IFileWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('file_display.pt',
        j01.form.interfaces.IFileWidget,
        z3c.form.interfaces.DISPLAY_MODE)

    # multi widget
    zope.component.provideAdapter(
        j01.form.widget.multi.getMultiFieldWidget,
        [zope.schema.interfaces.IDict,
         zope.schema.interfaces.IField,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.multi.getMultiFieldWidgetDispatcher,
        [zope.schema.interfaces.IList,
         zope.schema.interfaces.IField,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.multi.getMultiFieldWidgetDispatcher,
        [zope.schema.interfaces.ITuple,
         zope.schema.interfaces.IField,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    provideWidgetTemplate('multi_input.pt',
        j01.form.interfaces.IMultiWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('multi_display.pt',
        j01.form.interfaces.IMultiWidget,
        z3c.form.interfaces.DISPLAY_MODE)
    provideWidgetTemplate('multi_hidden.pt',
        j01.form.interfaces.IMultiWidget,
        z3c.form.interfaces.HIDDEN_MODE)

    # password
    zope.component.provideAdapter(
        j01.form.widget.password.PasswordConfirmationDataConverter)
    zope.component.provideAdapter(
        j01.form.widget.password.getPasswordWidget,
        [zope.schema.interfaces.IPassword,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.password.PasswordRequiredValue)
    provideWidgetTemplate('password_input.pt',
        j01.form.interfaces.IPasswordWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('password_display.pt',
        j01.form.interfaces.IPasswordWidget,
        z3c.form.interfaces.DISPLAY_MODE)
    provideWidgetTemplate('password_confirmation_input.pt',
        j01.form.interfaces.IPasswordConfirmationWidget,
        z3c.form.interfaces.INPUT_MODE)

    # radio
    zope.component.provideAdapter(
        j01.form.widget.radio.getRadioWidget,
        [zope.schema.interfaces.IBool,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    provideWidgetTemplate('radio_input.pt',
        j01.form.interfaces.IRadioWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('radio_display.pt',
        j01.form.interfaces.IRadioWidget,
        z3c.form.interfaces.DISPLAY_MODE)
    provideWidgetTemplate('radio_hidden.pt',
        j01.form.interfaces.IRadioWidget,
        z3c.form.interfaces.HIDDEN_MODE)
    provideWidgetTemplate('radio_input.pt',
        j01.form.widget.radio.RadioInlineWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('radio_picker_input.pt',
        j01.form.widget.radio.RadioPickerWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('radio_inline_picker_input.pt',
        j01.form.widget.radio.RadioInlinePickerWidget,
        z3c.form.interfaces.INPUT_MODE)

    # select
    zope.component.provideAdapter(
        j01.form.widget.select.ChoiceWidgetDispatcher)
    zope.component.provideAdapter(
        j01.form.widget.select.SelectFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.select.CollectionSelectFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.select.CollectionChoiceSelectFieldWidget)
    provideWidgetTemplate('select_input.pt',
        j01.form.interfaces.ISelectWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('select_display.pt',
        j01.form.interfaces.ISelectWidget,
        z3c.form.interfaces.DISPLAY_MODE)
    provideWidgetTemplate('select_hidden.pt',
        j01.form.interfaces.ISelectWidget,
        z3c.form.interfaces.HIDDEN_MODE)
    provideWidgetTemplate('select_optgroup_input.pt',
        j01.form.widget.select.GroupSelectWidget,
        z3c.form.interfaces.INPUT_MODE)

    # text
    zope.component.provideAdapter(
        j01.form.widget.text.getDateWidget,
        [zope.schema.interfaces.IDate,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getDatetimeWidget,
        [zope.schema.interfaces.IDatetime,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getTimeWidget,
        [zope.schema.interfaces.ITime,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getURLWidget,
        [zope.schema.interfaces.IURI,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getTextWidget,
        [zope.schema.interfaces.IBytesLine,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getTextWidget,
        [zope.schema.interfaces.IASCIILine,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getTextWidget,
        [zope.schema.interfaces.ITextLine,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getTextWidget,
        [zope.schema.interfaces.IId,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getTextWidget,
        [zope.schema.interfaces.IInt,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getTextWidget,
        [zope.schema.interfaces.IFloat,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getTextWidget,
        [zope.schema.interfaces.IDecimal,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getTextWidget,
        [zope.schema.interfaces.ITimedelta,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getColorWidget,
        [p01.schema.interfaces.IColor,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getDateWidget,
        [p01.schema.interfaces.IDate,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getDatetimeWidget,
        [p01.schema.interfaces.IDatetime,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getDatetimeLocalWidget,
        [p01.schema.interfaces.IDatetimeLocal,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getDatetimeLocalWidget,
        [p01.schema.interfaces.IDatetimeLocal,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getEMailWidget,
        [p01.schema.interfaces.IEMail,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getMonthWidget,
        [p01.schema.interfaces.IMonth,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getNumberWidget,
        [p01.schema.interfaces.INumber,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getSearchWidget,
        [p01.schema.interfaces.ISearch,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getTelWidget,
        [p01.schema.interfaces.ITel,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getTimeWidget,
        [p01.schema.interfaces.ITime,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getURLWidget,
        [p01.schema.interfaces.IURL,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.text.getWeekWidget,
        [p01.schema.interfaces.IWeek,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)

    provideWidgetTemplate('text_input.pt',
        j01.form.interfaces.ITextWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('text_display.pt',
        j01.form.interfaces.ITextWidget,
        z3c.form.interfaces.DISPLAY_MODE)
    provideWidgetTemplate('text_hidden.pt',
        j01.form.interfaces.ITextWidget,
        z3c.form.interfaces.HIDDEN_MODE)

    # textarea
    zope.component.provideAdapter(
        j01.form.widget.textarea.getTextAreaWidget,
        [zope.schema.interfaces.IASCII,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    zope.component.provideAdapter(
        j01.form.widget.textarea.getTextAreaWidget,
        [zope.schema.interfaces.IText,
         j01.form.layer.IFormLayer], z3c.form.interfaces.IFieldWidget)
    provideWidgetTemplate('textarea_input.pt',
        j01.form.interfaces.ITextAreaWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('textarea_display.pt',
        j01.form.interfaces.ITextAreaWidget,
        z3c.form.interfaces.DISPLAY_MODE)
    provideWidgetTemplate('textarea_hidden.pt',
        j01.form.interfaces.ITextAreaWidget,
        z3c.form.interfaces.HIDDEN_MODE)

    # textlines
    provideWidgetTemplate('textlines_input.pt',
        j01.form.interfaces.ITextLinesWidget,
        z3c.form.interfaces.INPUT_MODE)
    provideWidgetTemplate('textlines_display.pt',
        j01.form.interfaces.ITextLinesWidget,
        z3c.form.interfaces.DISPLAY_MODE)

    # widget ayout
    fPath = getPath('widget_layout_display.pt')
    layout = z3c.form.widget.WidgetLayoutFactory(fPath, 'text/html')
    zope.interface.directlyProvides(layout,
        z3c.form.interfaces.IWidgetLayoutTemplate)
    zope.component.provideAdapter(layout,
        (None, j01.form.layer.IFormLayer, None, None,
         j01.form.interfaces.IWidget),
        z3c.form.interfaces.IWidgetLayoutTemplate,
        name=z3c.form.interfaces.DISPLAY_MODE)
    fPath = getPath('widget_layout_input.pt')
    layout = z3c.form.widget.WidgetLayoutFactory(fPath, 'text/html')
    zope.interface.directlyProvides(layout,
        z3c.form.interfaces.IWidgetLayoutTemplate)
    zope.component.provideAdapter(layout,
        (None, j01.form.layer.IFormLayer, None, None,
         j01.form.interfaces.IWidget),
        z3c.form.interfaces.IWidgetLayoutTemplate,
        name=z3c.form.interfaces.INPUT_MODE)


def setUp(test=None):
    z3c.form.testing.setUp(test)
    setupFormDefaults()


def tearDown(test=None):
    z3c.form.testing.tearDown(test)



##############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH.
# All Rights Reserved.
#
##############################################################################
"""Form classes
$Id: form.py 4259 2015-06-14 14:30:24Z roger.ineichen $
"""
__docformat__ = 'restructuredtext'

import sys

import zope.interface
import zope.component
import zope.i18nmessageid

import z3c.form.form
import z3c.form.error
import z3c.form.button
import z3c.form.interfaces
from z3c.template.template import getLayoutTemplate
from z3c.template.template import getPageTemplate

import j01.form
from j01.form import btn
from j01.form import interfaces

_ = zope.i18nmessageid.MessageFactory('p01')


applyChanges = z3c.form.form.applyChanges


# supports z3c.form and j01.jsonrpc button and handlers
def extends(*args, **kwargs):
    """Copy form button, handler and fields from given form

    Note: this method supports both (z3c.form and j01.jsonrpc) concepts and
    uses the correct button and handlers.
    """
    frame = sys._getframe(1)
    f_locals = frame.f_locals
    if not kwargs.get('ignoreFields', False):
        f_locals['fields'] = z3c.form.field.Fields()
        for arg in args:
            f_locals['fields'] += getattr(arg, 'fields',
                z3c.form.field.Fields())
    if not kwargs.get('ignoreButtons', False):
        f_locals['buttons'] = btn.Buttons()
        for arg in args:
            f_locals['buttons'] += getattr(arg, 'buttons', btn.Buttons())
    if not kwargs.get('ignoreHandlers', False):
        f_locals['handlers'] = btn.Handlers()
        for arg in args:
            f_locals['handlers'] += getattr(arg, 'handlers', btn.Buttons())



###############################################################################
#
# browser request form classes

class IFormButtons(zope.interface.Interface):
    """Form buttons"""

    add = btn.Button(
        title=_(u"Add"),
        css='btn btn-add',
        )

    apply = btn.Button(
        title=_(u"Apply"),
        css='btn btn-apply',
        )

    cancel = btn.Button(
        title=_(u"Cancel"),
        css='btn btn-cancel',
        )


@zope.interface.implementer(interfaces.IForm)
class Form(j01.form.FormMixin, z3c.form.form.Form):
    """Simple form"""


@zope.interface.implementer(interfaces.IDisplayForm)
class DisplayForm(j01.form.FormMixin, z3c.form.form.Form):
    """Form for displaying fields (supporting buttons too)"""

    mode = z3c.form.interfaces.DISPLAY_MODE
    ignoreRequest = True


@zope.interface.implementer(interfaces.IAddForm)
class AddForm(j01.form.AddFormMixin, z3c.form.form.AddForm):
    """Add form."""

    ignoreRequest = True
    showCancel = True

    @btn.buttonAndHandler(IFormButtons['add'])
    def handleAdd(self, action):
        self.doHandleAdd(action)

    @btn.buttonAndHandler(IFormButtons['cancel'],
        condition=lambda form: form.showCancel)
    def handleCancel(self, action):
        self.doHandleCancel(action)


@zope.interface.implementer(interfaces.IEditForm)
class EditForm(j01.form.EditFormMixin, z3c.form.form.EditForm):
    """Edit form"""

    showCancel = True

    @btn.buttonAndHandler(IFormButtons['apply'])
    def handleApply(self, action):
        self.doHandleApply(action)

    @btn.buttonAndHandler(IFormButtons['cancel'],
        condition=lambda form: form.showCancel)
    def handleCancel(self, action):
        self.doHandleCancel(action)


###############################################################################
#
# jsonrpc request form classes

class IJSONRPCButtons(zope.interface.Interface):

    add = btn.JSONRPCButton(
        title=_(u'Add'),
        css='btn btn-add',
        )

    apply = btn.JSONRPCButton(
        title=_(u'Apply'),
        css='btn btn-apply',
        )

    cancel = btn.JSONRPCButton(
        title=_(u'Cancel'),
        css='btn btn-cancel',
        )


@zope.interface.implementer(interfaces.IJSONRPCForm)
class JSONRPCForm(j01.form.JSONRPCMixin, j01.form.FormMixin,
    z3c.form.form.Form):
    """JSONRPC form mixin."""


@zope.interface.implementer(interfaces.IJSONRPCAddForm)
class JSONRPCAddForm(j01.form.JSONRPCMixin, j01.form.AddFormMixin,
    z3c.form.form.AddForm):
    """JSONRPC add form."""

    showCancel = True

    @btn.buttonAndHandler(IJSONRPCButtons['add'])
    def handleAdd(self, action):
        self.doHandleAdd(action)

    @btn.buttonAndHandler(IJSONRPCButtons['cancel'],
        condition=lambda form: form.showCancel)
    def handleCancel(self, action):
        self.doHandleCancel(action)


@zope.interface.implementer(interfaces.IJSONRPCEditForm)
class JSONRPCEditForm(j01.form.JSONRPCMixin, j01.form.EditFormMixin,
    z3c.form.form.EditForm):
    """JSONRPC edit form."""

    showCancel = True

    @btn.buttonAndHandler(IJSONRPCButtons['apply'])
    def handleApply(self, action):
        self.doHandleApply(action)

    @btn.buttonAndHandler(IJSONRPCButtons['cancel'],
        condition=lambda form: form.showCancel)
    def handleCancel(self, action):
        self.doHandleCancel(action)


###############################################################################
#
# dialog form classes

class IDialogButtons(zope.interface.Interface):
    """Dialog form buttons"""

    add = btn.DialogButton(
        title=_(u'Add'),
        css='btn btn-add',
        )

    applyChanges = btn.DialogButton(
        title=_(u'Apply'),
        css='btn btn-apply',
        )

    cancel = btn.DialogButton(
        title=_(u'Cancel'),
        css='btn btn-cancel',
        )

    close = btn.DialogCloseButton(
        title=_(u'Close'),
        css='btn btn-close',
        )

    delete = btn.DialogButton(
        title=_(u'Delete'),
        css='btn btn-delete',
        )

    confirm = btn.DialogButton(
        title=_(u'Confirm'),
        css='btn btn-confirm',
        )


class DialogMixin(object):
    """Dialog mixin class"""

    zope.interface.implements(interfaces.IDialogForm)

    layout = getLayoutTemplate(name='dialog')
    template = getPageTemplate()

    prefix = 'dialog'

    j01DialogTitle = None
    closeDialog = False
    nextURL = None
    contentTargetExpression = None

    def setNextURL(self, url, status, closeDialog=True):
        """Helper for set a nextURL including status message and closeDialog"""
        self.closeDialog = closeDialog
        super(DialogMixin, self).setNextURL(url, status)

    def renderClose(self):
        """Return content if you need to render content after close."""
        return None

    def render(self):
        # knows what to return for the dialog parent
        if self.closeDialog:
            return self.renderClose()
        return self.template()


class DialogForm(DialogMixin, JSONRPCForm):
    """Dialog JSONRPC form."""

    zope.interface.implements(interfaces.IDialogForm)


class DialogEditForm(DialogMixin, JSONRPCEditForm):
    """Dialog JSONRPC edit form."""

    zope.interface.implements(interfaces.IDialogEditForm)

    closeOnApplyWithoutError = True

    def doHandleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        changes = self.applyChanges(data)
        if changes:
            self.status = self.successMessage
            # close on apply without error
            if self.closeOnApplyWithoutError:
                self.closeDialog = True
        else:
            self.status = self.noChangesMessage
            # close on apply without error
            if self.closeOnApplyWithoutError:
                self.closeDialog = True
        return changes

    def doHandleCancel(self, action):
        self.closeDialog = True

    @btn.buttonAndHandler(IDialogButtons['applyChanges'])
    def handleApply(self, action):
        self.doHandleApply(action)

    @btn.buttonAndHandler(IDialogButtons['cancel'])
    def handleCancel(self, action):
        self.doHandleCancel(action)


class DialogAddForm(DialogMixin, JSONRPCAddForm):
    """Dialog JSONRPC edit form."""

    zope.interface.implements(interfaces.IDialogAddForm)

    def doHandleAdd(self, action):
        # Note we, use the data from the request.form
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        obj = self.createAndAdd(data)
        if obj is not None:
            # mark only as finished if we get the new object
            self._finishedAdd = True
            self.closeDialog = True
        return obj

    def doHandleCancel(self, action):
        self.closeDialog = True

    @btn.buttonAndHandler(IDialogButtons['add'])
    def handleAdd(self, action):
        self.doHandleAdd(action)

    @btn.buttonAndHandler(IDialogButtons['cancel'])
    def handleCancel(self, action):
        self.doHandleCancel(action)


class DialogDeleteForm(DialogForm):
    """Dialog JSONRPC delete form."""

    zope.interface.implements(interfaces.IDialogDeleteForm)

    ignoreContext = True

    def doHandleDelete(self, action):
        raise NotImplementedError(
            'Subclass must implement doHandleDelete')

    def doHandleCancel(self, action):
        self.closeDialog = True

    @btn.buttonAndHandler(IDialogButtons['delete'])
    def handleDelete(self, action):
        self.doHandleDelete(action)

    @btn.buttonAndHandler(IDialogButtons['cancel'])
    def handleCancel(self, action):
        self.doHandleCancel(action)


class DialogConfirmForm(DialogForm):
    """Dialog JSONRPC confirm form."""

    zope.interface.implements(interfaces.IDialogConfirmForm)

    ignoreContext = True

    def doHandleConfirm(self, action):
        raise NotImplementedError('Subclass must implement doHandleConfirm')

    def doHandleCancel(self, action):
        self.closeDialog = True

    @btn.buttonAndHandler(IDialogButtons['confirm'])
    def handleConfirm(self, action):
        self.doHandleConfirm(action)

    @btn.buttonAndHandler(IDialogButtons['cancel'])
    def handleCancel(self, action):
        self.doHandleCancel(action)



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Text widget
$Id: gmap.py 4824 2018-04-24 09:33:55Z roger.ineichen $
"""
__docformat__ = 'restructuredtext'

import zope.interface

import z3c.form.interfaces
import z3c.form.widget

import m01.gmap.widget

from j01.form import interfaces
from j01.form.widget.widget import WidgetMixin


class GMapWidget(WidgetMixin, m01.gmap.widget.GMapWidget):
    """Text input type widget"""

    zope.interface.implementsOnly(interfaces.IGMapWidget)

    klass = u'gmap-control form-control'
    css = u'gmap'


class GeoPointGMapWidget(WidgetMixin, m01.gmap.widget.GeoPointGMapWidget):
    """Text input type widget"""

    zope.interface.implementsOnly(interfaces.IGeoPointGMapWidget)

    klass = u'gmap-point-control form-control'
    css = u'gmap-point'


# gmap
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getGMapWidget(field, request):
    """IFieldWidget factory for GMapWidget."""
    return z3c.form.widget.FieldWidget(field, GMapWidget(request))


@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getGeoPointGMapWidget(field, request):
    """IFieldWidget factory for GMapWidget."""
    return z3c.form.widget.FieldWidget(field, GeoPointGMapWidget(request))



###############################################################################
#
# Copyright (c) 2012 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Select ordered widget
$Id:$
"""
__docformat__ = 'restructuredtext'

import zope.interface
import zope.component
import zope.schema.interfaces
import zope.i18n
import zope.i18nmessageid

import z3c.form.interfaces
import z3c.form.widget

import j01.selectordered.widget

import j01.form.layer
from j01.form import interfaces
from j01.form.widget.widget import WidgetMixin

_ = zope.i18nmessageid.MessageFactory('p01')


class OrderedSelectWidget(WidgetMixin,
    j01.selectordered.widget.OrderedSelectWidget):
    """Ordered-Select widget implementation with JQuery template."""

    zope.interface.implementsOnly(interfaces.IOrderedSelectWidget)

    klass = u'select-ordered-control form-control'
    css = u'select-ordered'

    # btn labels
    @property
    def btnAddLabel(self):
        return zope.i18n.translate(_(u'add'), context=self.request)

    @property
    def btnRemoveLabel(self):
        return zope.i18n.translate(_(u'remove'), context=self.request)

    @property
    def btnUpLabel(self):
        return zope.i18n.translate(_(u'up'), context=self.request)

    @property
    def btnDownLabel(self):
        return zope.i18n.translate(_(u'down'), context=self.request)


# ordered select widget
@zope.component.adapter(zope.schema.interfaces.ISequence,
                        j01.form.layer.IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def OrderedSelectFieldWidget(field, request):
    """IFieldWidget factory for OrderedSelectWidget."""
    return z3c.form.widget.FieldWidget(field, OrderedSelectWidget(request))


@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def SequenceChoiceSelectFieldWidget(field, value_type, request):
    """IFieldWidget factory for OrderedSelectWidget."""
    return OrderedSelectFieldWidget(field, request)


# get
def getOrderedSelectWidget(field, request):
    """IFieldWidget factory for SelectWidget."""
    return z3c.form.widget.FieldWidget(field, OrderedSelectWidget(request))



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Select widget
$Id: select.py 4052 2014-05-24 15:30:23Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.component
import zope.interface
import zope.i18n
import zope.i18nmessageid
import zope.schema.interfaces

import z3c.form.interfaces
import z3c.form.widget
import z3c.form.browser.widget

from j01.form import interfaces
from j01.form.layer import IFormLayer
from j01.form.widget.widget import WidgetMixin

_ = zope.i18nmessageid.MessageFactory('p01')


class SelectWidget(WidgetMixin, z3c.form.browser.widget.HTMLSelectWidget,
    z3c.form.widget.SequenceWidget):
    """Select widget"""

    zope.interface.implementsOnly(interfaces.ISelectWidget)

    klass = u'select-control form-control'
    css = u'select'
    prompt = False

    noValueMessage = _('No value')
    promptMessage = _('Select a value ...')

    # Internal attributes
    _adapterValueAttributes = \
        z3c.form.widget.SequenceWidget._adapterValueAttributes + \
        ('noValueMessage', 'promptMessage', 'prompt')

    def isSelected(self, term):
        return term.token in self.value

    @property
    def items(self):
        if self.terms is None:  # update() has not been called yet
            return ()
        items = []
        if (not self.required or self.prompt) and self.multiple is None:
            message = self.noValueMessage
            if self.prompt:
                message = self.promptMessage
            items.append({
                'id': self.id + '-novalue',
                'value': self.noValueToken,
                'content': message,
                'selected': self.value == []
                })

        ignored = set(self.value)

        def addItem(idx, term, prefix=''):
            selected = self.isSelected(term)
            if selected and term.token in ignored:
                ignored.remove(term.token)
            id = '%s-%s%i' % (self.id, prefix, idx)
            content = term.token
            if zope.schema.interfaces.ITitledTokenizedTerm.providedBy(term):
                content = zope.i18n.translate(
                    term.title, context=self.request, default=term.title)
            items.append(
                {'id': id, 'value': term.token, 'content': content,
                 'selected': selected})

        for idx, term in enumerate(self.terms):
            addItem(idx, term)

        if ignored:
            # some values are not displayed, probably they went away from the vocabulary
            for idx, token in enumerate(sorted(ignored)):
                try:
                    term = self.terms.getTermByToken(token)
                except LookupError:
                    # just in case the term really went away
                    continue

                addItem(idx, term, prefix='missing-')
        return items


class MultiSelectWidget(SelectWidget):
    """Select widget implementation."""

    zope.interface.implementsOnly(interfaces.IMultiSelectWidget)

    prompt = False
    size = 5
    multiple = 'multiple'


# select picker, see http://silviomoreto.github.io/bootstrap-select/ for options
JAVASCRIPT = """
<script type="text/javascript">
$(document).ready(function(){
    $('#%(id)s').selectpicker();
});
</script>
"""


class SelectPickerOptions(object):
    """SelectPicker options"""

#    options = {
#        style: 'btn-default',
#        size: 'auto',
#        title: null,
#        selectedTextFormat : 'values',
#        noneSelectedText : 'Nothing selected',
#        noneResultsText : 'No results match',
#        countSelectedText: '{0} of {1} selected',
#        maxOptionsText: ['Limit reached ({n} {var} max)', 'Group limit reached ({n} {var} max)', ['items','item']],
#        width: false,
#        container: false,
#        hideDisabled: false,
#        showSubtext: false,
#        showIcon: true,
#        showContent: true,
#        dropupAuto: true,
#        header: false,
#        liveSearch: false,
#        actionsBox: false,
#        multipleSeparator: ', ',
#        iconBase: 'glyphicon',
#        tickIcon: 'glyphicon-ok',
#        maxOptions: false,
#        }

class SelectPickerWidget(SelectWidget):
    """SelectPickerWidget"""

    zope.interface.implementsOnly(interfaces.ISelectPickerWidget)

    klass = u'select-picker-control form-control'
    css = u'select-picker'

    @property
    def javascript(self):
        return JAVASCRIPT % {'id': self.id}


class MultiSelectPickerWidget(MultiSelectWidget):
    """MultiSelectPickerWidget"""

    zope.interface.implementsOnly(interfaces.IMultiSelectPickerWidget)

    klass = u'select-picker-control form-control'
    css = u'select-picker'

    @property
    def javascript(self):
        return JAVASCRIPT % {'id': self.id}


class GroupSelectWidget(SelectWidget):
    """Select widget with optgroup support"""

    zope.interface.implementsOnly(interfaces.IGroupSelectWidget)

    def appendSubTerms(self, groupName, items, subTerms, count):
        """Append collected sub terms as optgroup subItems"""
        subItems = []
        for subTerm in subTerms:
            id = '%s-%i' % (self.id, count)
            content = zope.i18n.translate(subTerm.title,
                context=self.request, default=subTerm.title)
            subItems.append(
                {'id': id,
                 'value': subTerm.token,
                 'content': content,
                 'selected': self.isSelected(subTerm)})
        items.append({'isGroup': True,
                      'content': groupName,
                      'subItems': subItems})

    @property
    def items(self):
        if self.terms is None:
            # update() has not been called yet
            return ()
        items = []
        if (not self.required or self.prompt) and self.multiple is None:
            message = self.noValueMessage
            if self.prompt:
                message = self.promptMessage
            items.append({
                'isGroup': False,
                'id': self.id + '-novalue',
                'value': self.noValueToken,
                'content': message,
                'selected': self.value == []
                })

        # setup optgroup and option items
        groupName = None
        subTerms = []
        for count, term in enumerate(self.terms):
            if term.isGroup:
                if groupName is not None and subTerms:
                    self.appendSubTerms(groupName, items, subTerms, count)
                     # set empty subTerms list
                    subTerms = []
                # set as next groupName
                groupName = zope.i18n.translate(term.title,
                    context=self.request, default=term.title)
            else:
                # collect sub item terms
                subTerms.append(term)

        # render the last collected sub terms with the latest groupName
        if groupName is not None:
            self.appendSubTerms(groupName, items, subTerms, count)

        return items


# adapter
@zope.component.adapter(zope.schema.interfaces.IChoice, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def ChoiceWidgetDispatcher(field, request):
    """Dispatch widget for IChoice based also on its source."""
    return zope.component.getMultiAdapter((field, field.vocabulary, request),
                                          z3c.form.interfaces.IFieldWidget)


@zope.component.adapter(zope.schema.interfaces.IChoice,
                        zope.interface.Interface, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def SelectFieldWidget(field, source, request):
    """IFieldWidget factory for SelectWidget."""
    return z3c.form.widget.FieldWidget(field, SelectWidget(request))


@zope.component.adapter(
    zope.schema.interfaces.IUnorderedCollection, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def CollectionSelectFieldWidget(field, request):
    """IFieldWidget factory for SelectWidget."""
    return z3c.form.widget.FieldWidget(field, MultiSelectWidget(request))


@zope.component.adapter(
    zope.schema.interfaces.IUnorderedCollection,
    zope.schema.interfaces.IChoice, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def CollectionChoiceSelectFieldWidget(field, value_type, request):
    """IFieldWidget factory for SelectWidget."""
    return z3c.form.widget.FieldWidget(field, SelectWidget(request))


# get
def getSelectWidget(field, request):
    """IFieldWidget factory for SelectWidget."""
    return z3c.form.widget.FieldWidget(field, SelectWidget(request))


def getMultiSelectWidget(field, request):
    """IFieldWidget factory for SelectWidget."""
    return z3c.form.widget.FieldWidget(field, MultiSelectWidget(request))


def getSelectPickerWidget(field, request):
    """IFieldWidget factory for SelectWidget."""
    return z3c.form.widget.FieldWidget(field, SelectPickerWidget(request))


def getMultiSelectPickerWidget(field, request):
    """IFieldWidget factory for SelectWidget."""
    return z3c.form.widget.FieldWidget(field, MultiSelectPickerWidget(request))


def getGroupSelectWidget(field, request):
    """IFieldWidget factory for SelectWidget."""
    return z3c.form.widget.FieldWidget(field, GroupSelectWidget(request))



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Exceptions

$Id: exceptions.py 3979 2014-03-25 10:59:26Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.schema

_ = zope.i18nmessageid.MessageFactory('p01')


# password confirmation
class PasswordComparsionError(zope.schema.ValidationError):
    __doc__ = _("""Password doesn't compare with confirmation value""")



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH
# All Rights Reserved.
#
###############################################################################
"""Buttons
$Id: btn.py 4054 2014-05-29 22:02:13Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import j01.dialog.btn
import j01.jsonrpc.btn


###############################################################################
#
# decorators

handler = j01.jsonrpc.btn.handler
buttonAndHandler = j01.jsonrpc.btn.buttonAndHandler


###############################################################################
#
# buttons, handlers and action

Buttons = j01.jsonrpc.btn.Buttons
Handlers = j01.jsonrpc.btn.Handlers
ButtonAction = j01.jsonrpc.btn.ButtonAction


###############################################################################
#
# browser buttons

Button = j01.jsonrpc.btn.Button
CloseButton = j01.jsonrpc.btn.CloseButton


###############################################################################
#
# jsonrpc buttons

JSButton = j01.jsonrpc.btn.JSButton
JSONRPCButton = j01.jsonrpc.btn.JSONRPCButton
JSONRPCClickButton = j01.jsonrpc.btn.JSONRPCClickButton
JSONRPCContentButton = j01.jsonrpc.btn.JSONRPCContentButton


###############################################################################
#
# dialog buttons

DialogButton = j01.dialog.btn.DialogButton
ShowDialogButton = j01.dialog.btn.ShowDialogButton
DialogContentButton = j01.dialog.btn.DialogContentButton
DialogCloseButton = j01.dialog.btn.DialogCloseButton


###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Text widget
$Id: rater.py 4844 2018-04-29 15:48:28Z roger.ineichen $
"""
__docformat__ = 'restructuredtext'

import zope.interface
import zope.component

import z3c.form.widget
import z3c.form.interfaces

import j01.rater.widget
import j01.rater.interfaces

from j01.form import interfaces
from j01.form.layer import IFormLayer
from j01.form.widget.widget import WidgetMixin


class RatingWidget(WidgetMixin, j01.rater.widget.RatingWidget):
    """RatingWidget widget"""

    zope.interface.implementsOnly(interfaces.IRatingWidget)

    klass = u'j01-rater-control j01-rater-five-star-control form-control'
    css = u'j01-rater'


class FiveStarRatingWidget(RatingWidget):
    """FiveStarRatingWidget widget"""

    zope.interface.implementsOnly(interfaces.IFiveStarRatingWidget)

    klass = u'j01-rater-control j01-rater-five-star-control form-control'
    css = u'j01-rater'

    increment = 1
    isHalfStar = False


class FiveHalfStarRatingWidget(RatingWidget):
    """FiveHalfStarRatingWidget widget"""

    zope.interface.implementsOnly(interfaces.IFiveHalfStarRatingWidget)

    klass = u'j01-rater-control j01-rater-five-half-star-control form-control'
    css = u'j01-rater'

    increment = 0.5
    isHalfStar = True


class FiveHalfStarFullRatingWidget(RatingWidget):
    """FiveHalfStarFullRatingWidget widget"""

    zope.interface.implementsOnly(interfaces.IFiveHalfStarFullRatingWidget)

    klass = u'j01-rater-control j01-rater-five-half-star-control form-control'
    css = u'j01-rater'

    increment = 1
    isHalfStar = True


# get
@zope.component.adapter(j01.rater.interfaces.IFiveStarRatingField, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getFiveStarRatingWidget(field, request):
    """IFieldWidget factory for IFiveStarRatingWidget."""
    return z3c.form.widget.FieldWidget(field, FiveStarRatingWidget(request))


@zope.component.adapter(j01.rater.interfaces.IFiveHalfStarRatingField,
    IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getFiveHalfStarRatingWidget(field, request):
    """IFieldWidget factory for IFiveHalfStarRatingWidget."""
    return z3c.form.widget.FieldWidget(field, FiveHalfStarRatingWidget(request))


@zope.component.adapter(j01.rater.interfaces.IFiveHalfStarFullRatingField,
    IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getFiveHalfStarFullRatingWidget(field, request):
    """IFieldWidget factory for IPasswordWidget."""
    return z3c.form.widget.FieldWidget(field,
        FiveHalfStarFullRatingWidget(request))



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Radio widget
$Id: radio.py 4806 2018-04-09 13:17:16Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import sys

import zope.i18n
import zope.i18nmessageid
import zope.component
import zope.interface
import zope.schema.interfaces
from zope.pagetemplate.interfaces import IPageTemplate

import z3c.form.interfaces
import z3c.form.widget
import z3c.form.util
import z3c.form.browser.widget

from j01.form import interfaces
from j01.form.layer import IFormLayer
from j01.form.widget.widget import WidgetMixin

_ = zope.i18nmessageid.MessageFactory('p01')

PY3 = sys.version_info[0] >= 3

try:
    unicode
except NameError:
    # Py3: Define unicode.
    unicode = str

def toUnicode(obj):
    if isinstance(obj, bytes):
        return obj.decode('utf-8', 'ignore')
    if PY3:
        return str(obj)
    else:
        return unicode(obj)


class RadioWidget(WidgetMixin, z3c.form.browser.widget.HTMLInputWidget,
    z3c.form.widget.SequenceWidget):
    """RadioWidget using a div wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.IRadioWidget)

    klass = u'radio-control' # no form-control
    css = u'radio'
    noValueMessage = _('No value')
    showNoValueItem = False

    # Internal attributes
    _adapterValueAttributes = \
        z3c.form.widget.SequenceWidget._adapterValueAttributes + \
        ('noValueMessage',)

    def isChecked(self, term):
        return term.token in self.value

    def renderForValue(self, value):
        term = self.terms.getTermByToken(value)
        checked = self.isChecked(term)
        id = '%s-%i' % (self.id, list(self.terms).index(term))
        item = {'id': id, 'name': self.name, 'value': term.token,
                'checked': checked}
        template = zope.component.getMultiAdapter(
            (self.context, self.request, self.form, self.field, self),
            IPageTemplate, name=self.mode + '_single')
        return template(self, item)

    @property
    def items(self):
        items = []
        if not self.required and self.showNoValueItem:
            label = zope.i18n.translate(self.noValueMessage,
                context=self.request, default=self.noValueMessage)
            if not self.value:
                checked = True
            elif self.noValueToken in self.value:
                checked = True
            else:
                checked = False
            items.append({
                'id': self.id + '-novalue',
                'name': self.name,
                'value': self.noValueToken,
                'label': label,
                'checked': checked,
                })
        if self.terms is not None:
            for count, term in enumerate(self.terms):
                checked = self.isChecked(term)
                id = '%s-%i' % (self.id, count)
                if zope.schema.interfaces.ITitledTokenizedTerm.providedBy(term):
                    label = zope.i18n.translate(term.title, context=self.request,
                        default=term.title)
                else:
                    label = toUnicode(term.value)
                items.append({
                    'id': id,
                    'name': self.name,
                    'value': term.token,
                    'label': label,
                    'checked': checked,
                    })
        return items


class RadioInlineWidget(RadioWidget):
    """RadioWidget using a span wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.IRadioInlineWidget)


JAVASCRIPT = """
<script type="text/javascript">
$(document).ready(function(){
    $('input[name^="%s"]').picker();
});
</script>
"""


class RadioPickerWidget(RadioWidget):
    """RadioPickerWidget using a div wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.IRadioPickerWidget)

    klass = u'radio-picker-control' # no form-control
    css = u'radio-picker'

    @property
    def javascript(self):
        return JAVASCRIPT % self.name.replace('.', '\\\.')


class RadioInlinePickerWidget(RadioWidget):
    """RadioInlinePickerWidget using a span wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.IRadioInlinePickerWidget)

    klass = u'radio-picker-control'
    css = u'radio-inline-picker'

    @property
    def javascript(self):
        return JAVASCRIPT % self.name.replace('.', '\\\.')


# get
@zope.component.adapter(zope.schema.interfaces.IField, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getRadioWidget(field, request):
    """IFieldWidget factory for RadioWidget."""
    return z3c.form.widget.FieldWidget(field, RadioWidget(request))


@zope.component.adapter(zope.schema.interfaces.IField, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getRadioInlineWidget(field, request):
    """IFieldWidget factory for RadioWidget."""
    return z3c.form.widget.FieldWidget(field, RadioInlineWidget(request))


@zope.component.adapter(zope.schema.interfaces.IField, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getRadioPickerWidget(field, request):
    """IFieldWidget factory for RadioWidget."""
    return z3c.form.widget.FieldWidget(field, RadioPickerWidget(request))


@zope.component.adapter(zope.schema.interfaces.IField, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getRadioInlinePickerWidget(field, request):
    """IFieldWidget factory for RadioWidget."""
    return z3c.form.widget.FieldWidget(field, RadioInlinePickerWidget(request))


###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""File upload widget
$Id: file.py 3934 2014-03-17 07:38:52Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.component
import zope.interface
import zope.schema.interfaces

import z3c.form.interfaces
import z3c.form.widget
import z3c.form.browser.file

from j01.form import interfaces
from j01.form.layer import IFormLayer
from j01.form.widget.widget import WidgetMixin


class FileWidget(WidgetMixin, z3c.form.browser.file.FileWidget):
    """Input type text widget implementation."""

    zope.interface.implementsOnly(interfaces.IFileWidget)

    klass = u'file-control form-control'
    css = u'file'

    # Filename and headers attribute get set by ``IDataConverter`` to the widget
    # provided by the FileUpload object of the form.
    headers = None
    filename = None


# get
@zope.component.adapter(zope.schema.interfaces.IBytes, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getFileWidget(field, request):
    """IFieldWidget factory for IPasswordWidget."""
    return z3c.form.widget.FieldWidget(field, FileWidget(request))



###############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id:$
"""
__docformat__ = 'restructuredtext'

import zope.component
import zope.interface

from z3c.form.interfaces import IDataConverter
from z3c.form.interfaces import IValidator
from z3c.form.interfaces import IErrorViewSnippet
from z3c.jsonrpc.interfaces import IJSONRPCPublisher
from z3c.jsonrpc.interfaces import IJSONRPCRequest
from z3c.jsonrpc.publisher import MethodPublisher

from j01.form import interfaces
import j01.form.layer


class J01Validator(MethodPublisher):

    zope.component.adapts(interfaces.IForm, IJSONRPCRequest)

    zope.interface.implements(IJSONRPCPublisher)

    def j01Validate(self, id, value):
        """Validate the value for the witdget with the given DOM field id."""
        res = u'OK'
        errorView = None

        # get last part from id as fieldName
        names = id.split('-')
        fieldName = names[len(names)-1]
        # optimize setUpWidgetValidation in your form and only setup the
        # relevant widget if possible
        self.context.setUpWidgetValidation(name=fieldName)

        # get widget by fieldName
        widget = self.context.widgets.get(fieldName)
        if widget is not None:
            content = self.context.widgets.content
            form = self.context.widgets.form
            try:
                value = IDataConverter(widget).toFieldValue(value)
                validator = zope.component.getMultiAdapter(
                    (content, self.request, self.context,
                     getattr(widget, 'field', None), widget), IValidator)
                error = validator.validate(value)

            except (zope.schema.ValidationError, ValueError), error:
                errorView = zope.component.getMultiAdapter(
                    (error, self.request, widget, widget.field,
                     form, content), IErrorViewSnippet)
                errorView.update()

        if errorView is not None:
            res = errorView.render()
        return {'id':id, 'result':res}



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Widgt mixin classes shared in form and jsform

$Id:$
"""
__docformat__ = "reStructuredText"

import urllib

import zope.interface
import zope.schema.interfaces
import zope.i18nmessageid
import zope.event
import zope.lifecycleevent
from zope.publisher.interfaces import NotFound
from zope.traversing.browser import absoluteURL

import z3c.form.util
import z3c.form.error
import z3c.form.interfaces
from z3c.jsonrpc.interfaces import IMethodPublisher
from z3c.template.template import getPageTemplate
from z3c.template.template import getLayoutTemplate

import j01.jsonrpc
from j01.form import interfaces
from j01.form import btn

_ = zope.i18nmessageid.MessageFactory('p01')

REDIRECT_STATUS_CODES = (301, 302, 303)

_marker = object()


###############################################################################
#
# apply changes supporting ignoreContext

def applyChanges(form, content, data):
    changes = {}
    for name, field in form.fields.items():
        # If the field is not in the data, then go on to the next one
        try:
            newValue = data[name]
        except KeyError:
            continue
        # If the value is NOT_CHANGED, ignore it, since the widget/converter
        # sent a strong message not to do so.
        if newValue is z3c.form.interfaces.NOT_CHANGED:
            continue
        elif form.widgetIgnoreApplyChanges.get(name):
            # ignore applyChanges. This is usefull for support the default
            # applyChanges method and skip widgets where we don't provide an
            # adapter
            continue
        elif form.widgetIgnoreContexts.get(name):
            # ignore applyChanges. Because we declared ignore context and this
            # means we can't use applyChanges for widgets defined in
            # widgetIgnoreContexts
            continue
        elif z3c.form.util.changedField(field.field, newValue, context=content):
            # Only update the data, if it is different
            dm = zope.component.getMultiAdapter(
                (content, field.field), z3c.form.interfaces.IDataManager)
            dm.set(newValue)
            # Record the change using information required later
            changes.setdefault(dm.field.interface, []).append(name)
    return changes


###############################################################################
#
# enhanced form mixin class

class LayoutHelperMixin(object):
    """Layout helper mixin class

    The following attributes can get used for render css class and id for html
    and body elements. This allows you to control css rendering or javascript
    includes.

    You can simply implement your own logik for apply the right ids or css
    classes in your pages within this methods. All you have to do is to use
    them in your layout with something like:

        <!DOCTYPE html>
        <html i18n:domain="demo" lang="de"
              tal:attributes="lang request/lang|nothing;
                              id view/cssHTMLId;
                              class view/cssHTMLClass">
        <head>...</head>
        <body tal:attributes="id view/cssBodyId;
                              class view/cssBodyClass">
        ...
        </body>
        </html>

    and apply the classes and ids like:

        cssBodyId = 'body'
        cssBodyClass = 'option1 feature2'
        cssHTMLId = 'layout2'
        cssHTMLClass = 'more another'

    """

    cssBodyId = None
    cssBodyClass = None
    cssHTMLId = None
    cssHTMLClass = None


class FormMixin(LayoutHelperMixin, j01.jsonrpc.HistoryStateMixin):
    """Form widgets mixin class"""

    template = getPageTemplate()
    layout = getLayoutTemplate()

    # simplified button and action handler concept (supports jsonrpc too!)
    buttons = btn.Buttons()
    handlers = btn.Handlers()

    formErrorsMessage = _('There were some errors.')
    successMessage = _('Data successfully updated.')
    noChangesMessage = _('No changes were applied.')

    # cached urls
    _contextURL = None
    # action/page ur
    _pageURL = None
    # next url
    nextURL = None
    # support download (direct) results
    directResult = None

    # default values used by IValue(name='default') adapter
    defaultValues = None

    # allows to skip action and widget processing. This is sometimes required
    # for JSON-RPC forms
    skipActions = False
    skipWidgets = False

    # set this conditions in your action handler method if needed
    # widgets normaly not change their value
    refreshWidgets = False
    # action condition may have changed after action execution
    refreshActions = False

    # override button prefix if you need to load different forms using the same
    # button in one single page.
    btnPrefix = 'buttons'

    # override widget prefix if you need to load different forms using the same
    # field names in one single page.
    widgetPrefix = 'widgets'

    # widget factories
    widgetFactories = {}

    # widget labels
    widgetLabels = {}
    widgetShowLabels = {}

    # widget titles
    widgetTitles = {}

    # widget descriptions
    widgetDescriptions = {}

    # required widgets
    widgetRequireds = {}
    widgetShowRequireds = {}

    # ignore required on validation
    ignoreRequiredOnValidations = {}

    # ignoreContext widgets
    widgetIgnoreContexts = {}

    # ignoreContext widgets
    widgetIgnoreApplyChanges = {}

    # ignoreContext widgets
    widgetIgnoreRequests = {}

    # widget modes
    widgetModes = {}

    # widget layout
    widgetInputLayoutName = z3c.form.interfaces.INPUT_MODE
    widgetDisplayLayoutName = z3c.form.interfaces.DISPLAY_MODE
    widgetHiddenLayoutName = z3c.form.interfaces.HIDDEN_MODE
    widgetInputLayoutNames = {}
    widgetDisplayLayoutNames = {}
    widgetHiddenLayoutNames = {}

    # widget placeholders
    widgetPlaceholders = {}

    # addons
    widgetAddOnWrapperDefault = unicode(
        '<div class="%(class)s">%(widget)s</div>')
    widgetAddOnWrappers = {}
    widgetBeforeAddOns = {}
    widgetAfterAddOns = {}

    # widget patterns
    widgetPatterns = {}

    # tab indexes
    tabIndexes = {}

    @property
    def name(self):
        """Never use an empty name if prefix is an empty string"""
        return self.prefix.strip('.') or 'form'

    @property
    def action(self):
        """Take care on action url."""
        return self.pageURL

    @property
    def contextURL(self):
        """Setup and cache context URL"""
        if self._contextURL is None:
            try:
                self._contextURL = absoluteURL(self.context, self.request)
            except TypeError:
                # insufficient context on error view
                pass
        return self._contextURL

    @property
    def pageURL(self):
        """Setup and cache context URL"""
        if self._pageURL is None:
            try:
                self._pageURL = '%s/%s' % (absoluteURL(self.context,
                    self.request), self.__name__)
            except TypeError:
                # insufficient context on error view
                pass
        return self._pageURL

    def setUpWidgetFields(self):
        """Setup additional/conditional widget fields"""
        pass

    def applyWidgetFactories(self):
        """Apply widget factories

        HEADSUP: if you switch modes based on application logic, you must
        setup the fields as self.fields in setUpWidgetFields. This is because
        the fields manager is a class instance and this switch is global for
        all form instances.

        NOTE: self.fields = self.fields.copy() does not work because it's
        broken. It will copy the field instances located inside the Fields
        instance as is.
        """
        for name, factory in self.widgetFactories.items():
            try:
                self.fields[name].widgetFactory = factory
            except KeyError:
                pass

    def applyWidgetModes(self):
        """Apply widget mode

        HEADSUP: if you switch modes based on application logic, you must
        setup the fields as self.fields in setUpWidgetFields. This is because
        the fields manager is a class instance and this switch is global for
        all form instances.

        NOTE: self.fields = self.fields.copy() does not work because it's
        broken. It will copy the field instances located inside the Fields
        instance as is.
        """
        for name, mode in self.widgetModes.items():
            try:
                self.fields[name].mode = mode
            except KeyError:
                pass

    def applyIgnoreContext(self):
        """Apply ignore context marker

        HEADSUP: if you switch modes based on application logic, you must
        setup the fields as self.fields in setUpWidgetFields. This is because
        the fields manager is a class instance and this switch is global for
        all form instances.

        NOTE: self.fields = self.fields.copy() does not work because it's
        broken. It will copy the field instances located inside the Fields
        instance as is.
        """
        for name, state in self.widgetIgnoreContexts.items():
            try:
                self.fields[name].ignoreContext = state
            except KeyError:
                pass

    def applyDefaultValues(self):
        """Apply default widget values

        Put those default values into self.defaultValues.

        NOTE: With z3c.form it's a NOGO, to just assign widget.value after
        update because some widgets need updating after that and the update
        gets a new value. The right way is to use an IValue(name='default')
        adapter and let widget.update() get the default value.
        """
        pass

    def applyWidgetLabels(self):
        """Apply widget labels"""
        for name, label in self.widgetLabels.items():
            try:
                self.widgets[name].label = label
            except KeyError:
                pass

    def applyWidgetShowLabels(self):
        """Apply widget labels"""
        for name, showLabel in self.widgetShowLabels.items():
            try:
                self.widgets[name].showLabel = showLabel
            except (KeyError, AttributeError):
                pass

    def applyWidgetTitles(self):
        """Apply widget titles"""
        for name, title in self.widgetTitles.items():
            try:
                self.widgets[name].title = title
            except KeyError:
                pass

    def applyWidgetDescriptions(self):
        """Apply widget descriptions"""
        for name, description in self.widgetDescriptions.items():
            try:
                widget = self.widgets[name]
                if description is True:
                    # use field description if value is True
                    description = widget.field.description
                widget.description = description
            except KeyError:
                pass

    def applyWidgetShowRequireds(self):
        """Apply widget showRequired"""
        # override required
        for name, showRequired in self.widgetShowRequireds.items():
            try:
                self.widgets[name].showRequired = showRequired
            except (KeyError, AttributeError):
                pass

    def applyWidgetPlaceholders(self):
        """Apply widget patterns"""
        for name, placeholder in self.widgetPlaceholders.items():
            try:
                self.widgets[name].placeholder = placeholder
            except KeyError:
                pass

    def applyWidgetPatterns(self):
        """Apply widget patterns"""
        for name, pattern in self.widgetPatterns.items():
            try:
                self.widgets[name].pattern = pattern
            except KeyError:
                pass

    def applyWidgetAddOns(self):
        """Apply widget addon before widget"""
        for name, content in self.widgetAddOnWrappers.items():
            try:
                self.widgets[name].addOnWrapper = content
            except KeyError:
                pass
        for name, content in self.widgetBeforeAddOns.items():
            try:
                self.widgets[name].addOnBefore = content
                if self.widgets[name].addOnWrapper is None:
                    self.widgets[name].addOnWrapper = \
                        self.widgetAddOnWrapperDefault
            except KeyError:
                pass
        for name, content in self.widgetAfterAddOns.items():
            try:
                self.widgets[name].addOnAfter = content
                if self.widgets[name].addOnWrapper is None:
                    self.widgets[name].addOnWrapper = \
                        self.widgetAddOnWrapperDefault
            except KeyError:
                pass

    def applyTabIndexes(self):
        """Apply tab indexes"""
        for name, idx in self.tabIndexes.items():
            try:
                self.widgets[name].tabindex = idx
            except KeyError:
                pass

    def applyWidgetLayouts(self):
        """Apply widget layout templates"""
        for widget in self.widgets.items():
            if widget.mode == z3c.form.interfaces.INPUT_MODE:
                name = self.widgetInputLayoutNames.get(widget.__name__)
                if name is None:
                    name = self.widgetInputLayoutName
            elif widget.mode == z3c.form.interfaces.DISPLAY_MODE:
                name = self.widgetDisplayLayoutName.get(widget.__name__)
                if name is None:
                    name = self.widgetDisplayLayoutName
            elif widget.mode == z3c.form.interfaces.HIDDEN_MODE:
                name = self.widgetHiddenLayoutNames.get(widget.__name__)
                if name is None:
                    name = self.widgetHiddenLayoutName
            else:
                raise ValueError("Unknown widget mode given", widget.mode)
            layout = zope.component.queryMultiAdapter(
                (self.context, self.request, self, widget.field, widget),
                z3c.form.interfaces.IWidgetLayoutTemplate, name=name)
            if layout is not None:
                widget.layout = layout

    def setUpActions(self):
        """Allows to override setUpActions and re-use super updateActions"""
        if self.btnPrefix is not None:
            # overrride button prefix before update actions
            self.buttons.prefix = self.btnPrefix
        super(FormMixin, self).updateActions()

    def setUpWidgets(self, prefix=None):
        """Allows to override setUpWidgets and re-use updateWidgets"""
        prefix = prefix and prefix or self.widgetPrefix
        # setup additional widgets
        self.setUpWidgetFields()
        # apply conditions
        self.applyWidgetFactories()
        self.applyWidgetModes()
        self.applyIgnoreContext()
        self.applyDefaultValues()
        # update widgets
        self.updateWidgets(prefix)
        # apply rendering conditions after widget update
        self.applyWidgetShowLabels()
        self.applyWidgetShowRequireds()
        self.applyWidgetLabels()
        self.applyWidgetTitles()
        self.applyWidgetDescriptions()
        self.applyWidgetPlaceholders()
        self.applyWidgetPatterns()
        self.applyWidgetAddOns()
        self.applyTabIndexes()

    def setUpWidgetValidation(self, name):
        """Support for single widget setup used by j01.validate"""
        # by default we simply setup al widgets. Customize this method in your
        # own form and only setup the relevant widget
        self.setUpWidgets()

    def setUpErrorViewSnippet(self, field, error, errors, setErrors=True):
        """Widget error setup helper method"""
        if isinstance(error, basestring):
            error = zope.interface.Invalid(error)
        err = zope.component.getMultiAdapter((
            error, self.request,
            self.widgets[field],
            self.widgets[field].field,
            self, self.getContent()), z3c.form.interfaces.IErrorViewSnippet)
        err.update()
        self.widgets[field].error = err
        errors += (err,)
        if setErrors:
            self.widgets.errors = errors
        return errors

# XXX: adjust ignoreContext etc. or not?
    # def updateWidgets(self, prefix=None):
    #     '''See interfaces.IForm'''
    #     self.widgets = zope.component.getMultiAdapter(
    #         (self, self.request, self.getContent()), interfaces.IWidgets)
    #     if prefix is not None:
    #         self.widgets.prefix = prefix
    #     self.widgets.mode = self.mode
    #     self.widgets.ignoreContext = self.ignoreContext
    #     self.widgets.ignoreRequest = self.ignoreRequest
    #     self.widgets.ignoreReadonly = self.ignoreReadonly
    #     self.widgets.update()

# XXX: don't setup ignoreRequiredOnExtract or setup different, see
#      J01WidgetFields for validation

    # def extractData(self, setErrors=True):
    #     '''See interfaces.IForm'''
    #     self.widgets.setErrors = setErrors
    #     self.widgets.ignoreRequiredOnExtract = self.ignoreRequiredOnExtract
    #     data, errors = self.widgets.extract()
    #     zope.event.notify(DataExtractedEvent(data, errors, self))
    #     return data, errors

    # def extractData(self, setErrors=True):
    #     # extract data
    #     data, errors = super(FormMixin, self).extractData(setErrors)
    #     # validate additional required fields
    #     for name, required in self.widgetRequireds.items():
    #         setWidgetErrors = setErrors
    #         try:
    #             w = self.widgets[name]
    #             w.setErrors is not None:
    #                 setWidgetErrors = w.setErrors
    #         except KeyError:
    #             pass
    #         else:
    #             if required and not data.get(name):
    #                 error = zope.schema.interfaces.RequiredMissing(name)
    #                 errors = self.setUpErrorViewSnippet(name, error, errors,
    #                     setErrors=setWidgetErrors)
    #     return data, errors

    def extractData(self, setErrors=True):
        # extract data
        data, errors = super(FormMixin, self).extractData(setErrors)
        # validate additional required fields
        for name, required in self.widgetRequireds.items():
            try:
                w = self.widgets[name]
            except KeyError:
                pass
            else:
                if required and not data.get(name):
                    # ensure errors for required fields
                    error = zope.schema.interfaces.RequiredMissing(name)
                    errors = self.setUpErrorViewSnippet(name, error, errors,
                        setErrors=setErrors)
        return data, errors

    def executeActions(self):
        """Dispatch actions.execute call"""
        self.actions.execute()

    def update(self):
        """Update form

        The update process supports setUpActions and setUpWidgets instead of
        updateActions and updateWidgets. This allows us to override the
        action and widget setup methods and re-use the original super
        classes.

        The default z3c.form calles the following methods in update:

        self.updateWidgets()
        self.updateActions()
        self.actions.execute()
        if self.refreshActions:
            self.updateActions()

        We implemented the following coditions:

        - skipWidgets
        - skipActions
        - refreshActions (also supported by z3c.form)
        - refreshWidgets

        And we use the following widget and action setup. Note, we
        implemented setUpActions and delegate the call to updateActions:

        if not self.skipWidgets:
            # default False
            self.setUpWidgets()

        if not self.skipActions:
            # default False
            self.setUpActions()
            self.executeActions()

        if self.refreshActions:
            # default False
            self.setUpActions()

        if self.refreshWidgets:
            # default False
            self.setUpWidgets()

        This allows us to prepare the JSONRPC call setup and gives more
        controll for complex form setup. Also see J01FormProcessor in
        j01/jsform/jsonrpc.py

        """
        if not self.skipWidgets:
            # default False
            self.setUpWidgets()

        if not self.skipActions:
            # default False
            self.setUpActions()
            self.executeActions()

        if self.refreshActions:
            # default False
            self.setUpActions()

        if self.refreshWidgets:
            # default False
            self.setUpWidgets()

    def render(self):
        if self.nextURL is not None:
            return None
        return self.template()

    def __call__(self):
        self.update()
        if self.directResult is not None:
            # support returning csv and pdf results
            return self.directResult
        elif self.nextURL is not None:
            # use nextURL for redirect
            self.request.response.redirect(self.nextURL)
            # don't render on redirect
            return u''
        elif self.request.response.getStatus() in REDIRECT_STATUS_CODES:
            # don't render on redirect
            return u''
        else:
            # default rendering
            return self.layout()


class AddFormMixin(FormMixin):
    """Add form mixin class"""

    def createAndAdd(self, data):
        obj = self.create(data)
        zope.event.notify(zope.lifecycleevent.ObjectCreatedEvent(obj))
        # HEADSUP: The add method could return None if something fails. This
        # will implicit prevent to apply self._finishedAdd in doHandleAdd method
        return self.add(obj)

    def doHandleAdd(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        obj = self.createAndAdd(data)
        if obj is not None:
            # HEADSUP: mark only as finished if we get the new object
            self._finishedAdd = True
        return obj

    def doHandleCancel(self, action):
        self.ignoreRequest = True
        self.refreshActions = True
        self.refreshWidgets = True

    def renderAfterAdd(self):
        return self.template()

    def render(self):
        if self._finishedAdd:
            return self.renderAfterAdd()
        return super(AddFormMixin, self).render()


class EditFormMixin(FormMixin):
    """Edit form mixin class"""

    def applyChanges(self, data):
        content = self.getContent()
        changes = applyChanges(self, content, data)
        # ``changes`` is a dictionary; if empty, there were no changes
        if changes:
            # Construct change-descriptions for the object-modified event
            descriptions = []
            for interface, names in changes.items():
                descriptions.append(
                    zope.lifecycleevent.Attributes(interface, *names))
            # Send out a detailed object-modified event
            zope.event.notify(
                zope.lifecycleevent.ObjectModifiedEvent(content, *descriptions))
        return changes

    def doHandleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        changes = self.applyChanges(data)
        if changes:
            self.status = self.successMessage
        else:
            self.status = self.noChangesMessage
        return changes

    def doHandleCancel(self, action):
        self.ignoreRequest = True
        self.refreshActions = True
        self.refreshWidgets = True


class JSONRPCMixin(object):
    """JSONRPC form mixin class providing traversal concept

    Note: the IJSONRPCForm interfaces makes sure that a pagelet
    (content provider) adapter is available by default. This is usefull if you
    instanciate such classes without to lookup them as adapters registred
    with the pagelet directive.
    """

    inputEnterActionName = None # see inputEnterJavaScript

    # JSON-RPC javascript callback arguments
    # content target expression where the result get rendered in. By default
    # the built-in argument ``#content`` get used
    contentTargetExpression = None

    # JSON-RPC content render javascript method name
    # The method registered as j01ContentRenderMethod can get used if the
    # default contentTargetExpression concept is not enough. Register a
    # javascript method with the j01.jsonrpc.js method j01RegisterRenderCallback
    # and we will lookup such a method with the method j01GetRenderCallback
    # in the default onSuccess handler. Or just use your own onSuccess callback
    # handler and use the contentRenderMethodName for invoke whatever you need
    contentRenderMethodName = None

    # optional scrollTo expression where the callback method will scroll to
    # after rendering. The default implementation uses offset().top. Feel free
    # to implement a custom j01CallbackRegistry method which uses another
    # concept instead of add more callback arguments
    scrollToExpression = None
    scrollToOffset = None
    scrollToSpeed = None

    # the next URL where the jsonrpc callback method will redirect to using
    # window.location.href = response.nextURL
    nextURL = None
    # the nextHash will update the url witout to redirect
    nextHash = None

    # url used for load content via j01LoadContent method
    nextContentURL = None

    # browser history support
    # skip brwoser history state marker. Note: we allways apply a browser
    # history state if this is not True, even if we don't provide a stateTitle,
    # stateURL or stateCallbackName
    skipState = False
    # disable browser history url if page only provides partial content
    # Note: we use the pageURL as default stateURL, see stateURL method below
    skipStateURL = False
    # history state title (not supported by all browser history implementations)
    stateTitle = None
    # browser history callback name for load content based on history state
    stateCallbackName = 'j01RenderContent'

    def publishTraverse(self, request, name):
        """Allows jsonrpc method traversing"""
        view = zope.component.queryMultiAdapter((self, request), name=name)
        if view is None or not IMethodPublisher.providedBy(view):
            raise NotFound(self, name, request)
        return view

    @property
    def inputEnterJavaScript(self):
        """Enter button click handler.

        You can define an action handler name which get called on enter button
        call in your form like:

        inputEnterActionName = 'myHandlerName'

        Note: you need to include the inputEnter javascript in your template
        within:

        <script type="text/javascript"
                tal:content="view/inputEnterJavaScript">
        </script>
        """
        return self.buttons.getInputEnterJavaScript(self, self.request)


    def setNextURL(self, url, status):
        """Helper for set a nextURL including a status message

        Note: the status message must be an i18n message which will get
        translated later as status message.

        If you don't use a status message just use self.nextURL = myURL and
        don't use this method.

        """
        self.nextURL = '%s?%s' % (url, urllib.urlencode({'status':status}))

    @property
    def stateURL(self):
        """Browser history url

        Only use an url if the full page can get loaded within a browser request
        make sure that the page provides a layout within the browser request.
        Remember, a jsonrpc request will only call update/render and a browser
        request will call __call__/update/render. Only the __call__ method will
        include the layout template.
        """
        if not self.skipState and not self.skipStateURL:
            # only used if state and state url are not marked as skipped
            return self.pageURL


###############################################################################
#
# IValue adapter

class DefaultValueProvider(object):
    """Default value adapter"""

    def __init__(self, value):
        self.value = value

    def get(self):
        return self.value


@zope.interface.implementer(z3c.form.interfaces.IValue)
@zope.component.adapter(zope.interface.Interface, zope.interface.Interface,
    interfaces.IForm, zope.schema.interfaces.IField,
    z3c.form.interfaces.IWidget)
def getDefaultValueProvider(context, request, form, field, widget):
    """Returns a default value provider or None"""
    if form.defaultValues is None:
        return None

    name = widget.name
    while name:
        try:
            value = form.defaultValues[name]
            return DefaultValueProvider(value)
        except KeyError:
            parts = name.split('.')[1:]
            name = '.'.join(parts)


##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Field Implementation

$Id: field.py 4644 2017-09-17 23:14:29Z roger.ineichen $
"""
__docformat__ = "reStructuredText"
import zope.component
import zope.interface
import zope.location
import zope.schema.interfaces

import z3c.form.util
import z3c.form.interfaces
from z3c.form.error import MultipleErrors
from z3c.form.widget import AfterWidgetUpdateEvent

from j01.form import interfaces


@zope.interface.implementer_only(z3c.form.interfaces.IWidgets)
class J01FieldWidgets(z3c.form.util.Manager):
    """Widget manager for IFieldWidget."""

    zope.component.adapts(interfaces.IForm, z3c.form.interfaces.IFormLayer,
        zope.interface.Interface)

    prefix = 'widgets.'
    mode = z3c.form.interfaces.INPUT_MODE
    errors = ()
    hasRequiredFields = False
    ignoreContext = False
    ignoreRequest = False
    ignoreReadonly = False
    ignoreRequiredOnExtract = False
    setErrors = True

    def __init__(self, form, request, content):
        super(J01FieldWidgets, self).__init__()
        self.form = form
        self.request = request
        self.content = content

    def validate(self, data):
        """Manager validator will validate invariants

        NOTE: this validator can only use the form.ignoreContext and will
        ignore the field.ignoreContext setup
        """
        fields = self.form.fields.values()

        # Step 1: Collect the data for the various schemas
        schemaData = {}
        for field in fields:
            schema = field.interface
            if schema is None:
                continue

            fieldData = schemaData.setdefault(schema, {})
            if field.__name__ in data:
                fieldData[field.field.__name__] = data[field.__name__]

        # Step 2: Validate the individual schemas and collect errors
        errors = ()
# XXX: I guess self.content is not the right context if we use an
#      adapted context. Change and use the adapted context as content for each
#      schema.
        content = self.content
        if self.ignoreContext:
            content = None
        for schema, fieldData in schemaData.items():
            validator = zope.component.getMultiAdapter(
                (content, self.request, self.form, schema, self),
                z3c.form.interfaces.IManagerValidator)
            errors += validator.validate(fieldData)

        return errors

    def update(self):
        """See interfaces.IWidgets"""
        # Create a unique prefix.
        prefix = z3c.form.util.expandPrefix(self.form.prefix)
        prefix += z3c.form.util.expandPrefix(self.prefix)
        # Walk through each field, making a widget out of it.
        uniqueOrderedKeys = []
        for field in self.form.fields.values():
            # Step 0. Determine whether the context should be ignored.
            ignoreContext = self.ignoreContext
            if field.ignoreContext is not None:
                ignoreContext = field.ignoreContext
            # Step 1: Determine the mode of the widget.
            mode = self.mode
            if field.mode is not None:
                mode = field.mode
            elif field.field.readonly and not self.ignoreReadonly:
                mode = z3c.form.interfaces.DISPLAY_MODE
            elif not ignoreContext:
                # get data manager
                dm = zope.component.getMultiAdapter((self.content, field.field),
                    z3c.form.interfaces.IDataManager)
                if not dm.canWrite():
                    # If we do not have enough permissions to write to the
                    # attribute, then switch to display mode.
                    mode = z3c.form.interfaces.DISPLAY_MODE
            # Step 2: Get the widget for the given field.
            shortName = field.__name__
            newWidget = True
            if shortName in self._data:
                # reuse existing widget
                widget = self._data[shortName]
                newWidget = False
            elif field.widgetFactory.get(mode) is not None:
                factory = field.widgetFactory.get(mode)
                widget = factory(field.field, self.request)
            else:
                widget = zope.component.getMultiAdapter(
                    (field.field, self.request),
                    z3c.form.interfaces.IFieldWidget)
            # Step 3: Set the prefix for the widget
            widget.name = prefix + shortName
            widget.id = (prefix + shortName).replace('.', '-')
            # Step 4: Set the context
            widget.context = self.content
            # Step 5: Set the form
            widget.form = self.form
            # Optimization: Set both interfaces here, rather in step 4 and 5:
            # ``alsoProvides`` is quite slow
            zope.interface.alsoProvides(widget,
                z3c.form.interfaces.IContextAware,
                z3c.form.interfaces.IFormAware)
            # Step 6: Set some variables
            widget.ignoreContext = ignoreContext
            # override widget.required if given otherwise use existing required
            widget.required = self.form.widgetRequireds.get(shortName,
                widget.required)
            widget.ignoreRequest = self.form.widgetIgnoreRequests.get(shortName,
                self.ignoreRequest)
            if field.showDefault is not None:
                widget.showDefault = field.showDefault
            # Step 7: Set the mode of the widget
            widget.mode = mode
            # Step 8: Setup ignore on validation used during extract validation
            widget.ignoreRequiredOnValidation = \
                self.form.ignoreRequiredOnValidations.get(shortName,
                    self.ignoreRequiredOnExtract)
            # Step 9: Update the widget
            widget.update()
            zope.event.notify(AfterWidgetUpdateEvent(widget))
            # Step 10: Add the widget to the manager
            if widget.required:
                self.hasRequiredFields = True
            uniqueOrderedKeys.append(shortName)
            if newWidget:
                self._data_values.append(widget)
                self._data[shortName] = widget
                zope.location.locate(widget, self, shortName)
            # always ensure that we add all keys and keep the order given from
            # button items
            self._data_keys = uniqueOrderedKeys

    def _extract(self, returnRaw=False):
        data = {}
        errors = ()
        for name, widget in self.items():
            if widget.mode == z3c.form.interfaces.DISPLAY_MODE:
                continue
            value = widget.field.missing_value
            try:
                widget.setErrors = self.setErrors
                raw = widget.extract()
                if raw is not z3c.form.interfaces.NO_VALUE:
                    value = z3c.form.interfaces.IDataConverter(widget
                        ).toFieldValue(raw)
                # don't set ignoreRequiredOnValidation based on self, we provide
                # a per widget setup with j01.form ignoreRequiredOnValidations
                # and the update method already applied this to the widget
                # widget.ignoreRequiredOnValidation = \
                #     self.ignoreRequiredOnExtract
                zope.component.getMultiAdapter((
                    #self.content,
                    widget.context,
                    self.request,
                    self.form,
                    getattr(widget, 'field', None),
                    widget),
                    z3c.form.interfaces.IValidator).validate(value)
            except (zope.interface.Invalid,
                    ValueError, MultipleErrors) as error:
                view = zope.component.getMultiAdapter((
                    error,
                    self.request,
                    widget,
                    widget.field,
                    self.form,
                    #self.content
                    widget.context),
                    z3c.form.interfaces.IErrorViewSnippet)
                view.update()
                if self.setErrors:
                    widget.error = view
                errors += (view,)
            else:
                name = widget.__name__
                if returnRaw:
                    data[name] = raw
                else:
                    data[name] = value
        for error in self.validate(data):
            view = zope.component.getMultiAdapter((
                error,
                self.request,
                None, None,
                self.form,
                # self.content,
                widget.context),
                z3c.form.interfaces.IErrorViewSnippet)
            view.update()
            errors += (view,)
        if self.setErrors:
            self.errors = errors
        return data, errors

    def extract(self):
        """See interfaces.IWidgets"""
        return self._extract(returnRaw=False)

    def extractRaw(self):
        """See interfaces.IWidgets"""
        return self._extract(returnRaw=True)



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Dictionary widget
$Id: dictionary.py 4891 2018-06-05 12:32:20Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface

import z3c.form.widget
import z3c.form.converter
import z3c.form.browser.widget
import z3c.form.browser.textarea

from j01.form import interfaces
from j01.form.layer import IFormLayer
from j01.form.widget.widget import WidgetMixin


class DictKeyValueWidget(WidgetMixin, z3c.form.browser.textarea.TextAreaWidget):
    """Input type text widget implementation for dict with key/value values."""

    zope.interface.implementsOnly(interfaces.IDictKeyValueWidget)

    klass = u'dictionary-control form-control'
    css = u'dictionary'
    value = u''

    label = None

    def update(self):
        super(DictKeyValueWidget, self).update()
        z3c.form.browser.widget.addFieldClass(self)


# converter
class DictKeyValueConverter(z3c.form.converter.BaseDataConverter):
    """Data converter for IDictKeyValueWidget."""

    zope.component.adapts(
        zope.schema.interfaces.IDict, interfaces.IDictKeyValueWidget)

    def toWidgetValue(self, value):
        """Convert from text lines to HTML representation."""
        # if the value is the missing value, then an empty list is produced.
        if value is self.field.missing_value:
            return u''
        elif value is None:
            # in case we use missing_value = {}
            return u''
        res = u''
        for k, v in value.items():
            res += u'%s:%s\n' % (k, v)
        return res

    def toFieldValue(self, value):
        """See interfaces.IDataConverter"""
        widget = self.widget
        if not len(value):
            return self.field.missing_value

        # find key type
        keyType = self.field.key_type._type
        if keyType is None:
            keyType = unicode
        if isinstance(keyType, tuple):
            keyType = keyType[0]

        # find value type
        valueType = self.field.value_type._type
        if isinstance(valueType, tuple):
            valueType = valueType[0]

        mapping = {}
        for entry in value.splitlines():
            if not entry:
                # ignore leading and ending empty linebreaks
                continue
            k, v = entry.split(':')
            mapping[keyType(k.strip())] = valueType(v.strip())
        return mapping


def getDictKeyValueWidget(field, request):
    """IFieldWidget factory for DictKeyValueWidget."""

    return z3c.form.widget.FieldWidget(field, DictKeyValueWidget(request))


################################################################################
#
# proxy widget

class ProxyWidget(DictKeyValueWidget):
    """Proxy protocol:url widget with support for ":" in url:port"""

    zope.interface.implements(interfaces.IProxyWidget)


# converter
class ProxyWidgetConverter(DictKeyValueConverter):
    """Data converter for IProxyWidget"""

    zope.component.adapts(zope.schema.interfaces.IDict, interfaces.IProxyWidget)

    def toFieldValue(self, value):
        """See interfaces.IDataConverter"""
        widget = self.widget
        if not len(value):
            return self.field.missing_value

        # find key type
        keyType = self.field.key_type._type
        if keyType is None:
            keyType = unicode
        if isinstance(keyType, tuple):
            keyType = keyType[0]

        # find value type
        valueType = self.field.value_type._type
        if isinstance(valueType, tuple):
            valueType = valueType[0]

        mapping = {}
        for entry in value.splitlines():
            if not entry:
                # ignore leading and ending empty linebreaks
                continue
            parts = entry.split(':')
            if len(parts) == 2:
                k, v = parts
            elif len(parts) > 2:
                k = parts[0]
                v = ':'.join(parts[1:])
            else:
                # parts < 2, that's a bad value (missing : separator)
                return mapping
            mapping[keyType(k.strip())] = valueType(v.strip())
        return mapping


def getProxyWidget(field, request):
    """IFieldWidget factory for ProxyWidget."""
    return z3c.form.widget.FieldWidget(field, ProxyWidget(request))



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Checbox widget
$Id: checkbox.py 4535 2016-09-16 00:05:50Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.i18n
import zope.schema.interfaces
import zope.schema.vocabulary

import z3c.form.term
import z3c.form.widget
import z3c.form.browser.widget

from j01.form import interfaces
from j01.form.widget.widget import WidgetMixin


# checkbox widgets
class CheckBoxWidget(WidgetMixin, z3c.form.browser.widget.HTMLInputWidget,
    z3c.form.widget.SequenceWidget):
    """CheckBoxWidget using a div wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.ICheckBoxWidget)

    showLabel = True
    showItemLabel = True
    klass = u'checkbox-control' # no form-control
    css = u'checkbox'

    def isChecked(self, term):
        return term.token in self.value

    def getChecked(self, term):
        """Get checked marker for input element"""
        return term.token in self.value and 'checked' or None

    def getLabel(self, term):
        """Get label for term"""
        if zope.schema.interfaces.ITitledTokenizedTerm.providedBy(term):
            return zope.i18n.translate(term.title, context=self.request,
                              default=term.title)
        else:
            return unicode(term.value)

    @property
    def items(self):
        items = []
        append = items.append
        for count, term in enumerate(self.terms):
            append({
                'id': '%s-%i' % (self.id, count),
                'name': self.name + ':list',
                'value': term.token,
                'label': self.getLabel(term),
                'checked': self.getChecked(term),
                # allows to skip label, but don't use non label because the
                # label is used in the error view snippet too
                'showItemLabel': self.showItemLabel,
                })
        return items


class CheckBoxInlineWidget(CheckBoxWidget):
    """CheckBoxWidget using a span wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.ICheckBoxInlineWidget)

    showLabel = True
    showItemLabel = True
    klass = u'checkbox-inline-control' # no form-control
    css = u'checkbox-inline'


class SingleCheckBoxWidget(CheckBoxWidget):
    """SingleCheckBoxWidget using a div wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.ISingleCheckBoxWidget)

    showLabel = True
    showItemLabel = False
    klass = u'single-checkbox-control' # no form-control
    css = u'single-checkbox'

    def updateTerms(self):
        if self.terms is None:
            self.terms = z3c.form.term.Terms()
            self.terms.terms = zope.schema.vocabulary.SimpleVocabulary((
                zope.schema.vocabulary.SimpleTerm('selected', 'selected',
                                      self.label or self.field.title), ))
        return self.terms


class SingleCheckBoxInlineWidget(SingleCheckBoxWidget):
    """SingleCheckBoxWidget using a span wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.ISingleCheckBoxInlineWidget)

    showLabel = False
    showItemLabel = True
    klass = u'single-checkbox-inline-control' # no form-control
    css = u'single-checkbox-inline'



# checkbox picker widgets
JAVASCRIPT = """<script type="text/javascript">
$(document).ready(function(){
    $('input[name^="%s"]').picker();
});
</script>
"""


class CheckBoxPickerWidget(CheckBoxWidget):
    """CheckBoxPickerWidget using a div wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.ICheckBoxPickerWidget)

    showLabel = True
    showItemLabel = True
    klass = u'checkbox-picker-control'
    css = u'checkbox-picker'

    @property
    def javascript(self):
        return JAVASCRIPT % self.name.replace('.', '\\\.')


class CheckBoxInlinePickerWidget(CheckBoxPickerWidget):
    """RadioInlinePickerWidget using a span wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.ICheckBoxInlinePickerWidget)

    showLabel = True
    showItemLabel = True
    klass = u'checkbox-inline-picker-control'
    css = u'checkbox-inline-picker'

    @property
    def javascript(self):
        return JAVASCRIPT % self.name.replace('.', '\\\.')


# picker widgets
class SingleCheckBoxPickerWidget(SingleCheckBoxWidget):
    """SingleCheckBoxWidget using a div wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.ISingleCheckBoxPickerWidget)

    showLabel = True
    showItemLabel = False
    klass = u'single-checkbox-picker-control' # no form-control
    css = u'single-checkbox-picker'

    @property
    def javascript(self):
        return JAVASCRIPT % self.name.replace('.', '\\\.')


class SingleCheckBoxInlinePickerWidget(SingleCheckBoxPickerWidget):
    """SingleCheckBoxWidget using a span wrapper for option tags"""

    zope.interface.implementsOnly(interfaces.ISingleCheckBoxInlinePickerWidget)

    showLabel = False
    showItemLabel = True
    klass = u'single-checkbox-inline-picker-control' # no form-control
    css = u'single-checkbox-inline-picker'

    @property
    def javascript(self):
        return JAVASCRIPT % self.name.replace('.', '\\\.')


# get checbox widgets
def getCheckBoxWidget(field, request):
    """IFieldWidget factory for CheckBoxWidget."""
    return z3c.form.widget.FieldWidget(field, CheckBoxWidget(request))


def getCheckBoxInlineWidget(field, request):
    """IFieldWidget factory for CheckBoxInlineWidget."""
    return z3c.form.widget.FieldWidget(field, CheckBoxInlineWidget(request))


def getSingleCheckBoxWidget(field, request):
    """IFieldWidget factory for SingleCheckBoxWidget."""
    return z3c.form.widget.FieldWidget(field, SingleCheckBoxWidget(request))


def getSingleCheckBoxInlineWidget(field, request):
    """IFieldWidget factory for SingleCheckBoxInlineWidget."""
    return z3c.form.widget.FieldWidget(field,
        SingleCheckBoxInlineWidget(request))


# get checkbox picker widgets
def getCheckBoxPickerWidget(field, request):
    """IFieldWidget factory for CheckBoxWidget."""
    return z3c.form.widget.FieldWidget(field, CheckBoxPickerWidget(request))


def getCheckBoxInlinePickerWidget(field, request):
    """IFieldWidget factory for CheckBoxInlinePickerWidget."""
    return z3c.form.widget.FieldWidget(field,
        CheckBoxInlinePickerWidget(request))


def getSingleCheckBoxPickerWidget(field, request):
    """IFieldWidget factory for SingleCheckBoxWidget."""
    return z3c.form.widget.FieldWidget(field,
        SingleCheckBoxPickerWidget(request))


def getSingleCheckBoxInlinePickerWidget(field, request):
    """IFieldWidget factory for SingleCheckBoxInlineWidget."""
    return z3c.form.widget.FieldWidget(field,
        SingleCheckBoxInlinePickerWidget(request))



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Widgets
$Id: __init__.py 4891 2018-06-05 12:32:20Z roger.ineichen $
"""
__docformat__ = 'restructuredtext'

# checkbox
from j01.form.widget.checkbox import getCheckBoxWidget
from j01.form.widget.checkbox import getCheckBoxInlineWidget

from j01.form.widget.checkbox import getSingleCheckBoxWidget
from j01.form.widget.checkbox import getSingleCheckBoxInlineWidget

from j01.form.widget.checkbox import getCheckBoxPickerWidget
from j01.form.widget.checkbox import getCheckBoxInlinePickerWidget

from j01.form.widget.checkbox import getSingleCheckBoxPickerWidget
from j01.form.widget.checkbox import getSingleCheckBoxInlinePickerWidget

# dictionary
from j01.form.widget.dictionary import getDictKeyValueWidget
from j01.form.widget.dictionary import getProxyWidget

# file
from j01.form.widget.file import getFileWidget

# multi
from j01.form.widget.multi import getMultiFieldWidget

# password
from j01.form.widget.password import getPasswordWidget
from j01.form.widget.password import getPasswordConfirmationWidget

# radio
from j01.form.widget.radio import getRadioWidget
from j01.form.widget.radio import getRadioInlineWidget
from j01.form.widget.radio import getRadioPickerWidget
from j01.form.widget.radio import getRadioInlinePickerWidget

# select
from j01.form.widget.select import getSelectWidget
from j01.form.widget.select import getMultiSelectWidget
from j01.form.widget.select import getSelectPickerWidget
from j01.form.widget.select import getMultiSelectPickerWidget
from j01.form.widget.select import getGroupSelectWidget

# text and html5 variants
from j01.form.widget.text import getTextWidget
from j01.form.widget.text import getEMailWidget
from j01.form.widget.text import getDateWidget
from j01.form.widget.text import getDatetimeWidget
from j01.form.widget.text import getDatetimeLocalWidget
from j01.form.widget.text import getTimeWidget
from j01.form.widget.text import getWeekWidget
from j01.form.widget.text import getMonthWidget
from j01.form.widget.text import getColorWidget
from j01.form.widget.text import getSearchWidget
from j01.form.widget.text import getURLWidget
from j01.form.widget.text import getNumberWidget
from j01.form.widget.text import getTelWidget

# textlines
from j01.form.widget.textlines import getTextLinesWidget

# textarea
from j01.form.widget.textarea import getTextAreaWidget

# j01.datepicker
try:
    # available if j01.datepicker package is available
    from j01.form.widget.datepicker import getDatePickerWidget
    from j01.form.widget.datepicker import getStartDatePickerWidget
    from j01.form.widget.datepicker import getEndDatePickerWidget
except ImportError:
    pass

# j01.datetimepicker
try:
    # available if j01.datetimepicker package is available
    from j01.form.widget.datetimepicker import getDateTimePickerWidget
except ImportError:
    pass

# j01.rater
try:
    # available if j01.rater package is available
    from j01.form.widget.rater import getFiveStarRatingWidget
    from j01.form.widget.rater import getFiveHalfStarRatingWidget
    from j01.form.widget.rater import getFiveHalfStarFullRatingWidget
except ImportError:
    pass

# j01.select2
try:
    # available if j01.select2 package is available
    from j01.form.widget.select2 import getSelect2Widget
    from j01.form.widget.select2 import getSingleSelect2Widget
    from j01.form.widget.select2 import getTagListSelect2Widget
    from j01.form.widget.select2 import getLiveListSelect2Widget
except ImportError:
    pass

# m01.gmap
try:
    # available if m01.gmap package is available
    from j01.form.widget.gmap import getGMapWidget
    from j01.form.widget.gmap import getGeoPointGMapWidget
except ImportError:
    pass

# j01.selectordered
try:
    from j01.form.widget.selectordered import getOrderedSelectWidget
except ImportError:
    pass




##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Multi Widget Implementation

$Id: select.py 78513 2007-07-31 23:03:47Z srichter $
"""
__docformat__ = "reStructuredText"
from operator import attrgetter

import zope.component
import zope.interface
import zope.i18nmessageid

import z3c.form.interfaces
import z3c.form.widget
import z3c.form.browser.widget

_ = zope.i18nmessageid.MessageFactory('p01')

from j01.form import btn
from j01.form import interfaces
from j01.form.widget.widget import WidgetMixin


@zope.interface.implementer(z3c.form.interfaces.IButtonForm,
    z3c.form.interfaces.IHandlerForm)
class FormMixin(object):
    pass


class IMultiFormButtons(zope.interface.Interface):
    """Multi form buttons"""

    add = btn.Button(
        title=_(u"Add"),
        css='btn btn-add-multi',
        )

    remove = btn.Button(
        title=_(u"Remove"),
        css='btn btn-remove-multi',
        )


@zope.interface.implementer(interfaces.IMultiWidget)
class MultiWidget(WidgetMixin, z3c.form.browser.widget.HTMLFormElement,
    z3c.form.widget.MultiWidget, FormMixin):
    """Multi widget implementation."""

    buttons = btn.Buttons()

    prefix = 'widget'
    klass = u'multi-control' # no form-control, that's a form
    css = u'multi'
    items = ()

    # show labels
    showLabel = True
    # show labels for key widgets
    showKeyWidgetLabel = False
    # show labels for value widgets
    showWidgetLabel = False

    # Internal attributes
    _adapterValueAttributes = ('label', 'name', 'required', 'title',
        'showLabel')

    def updateActions(self):
        self.updateAllowAddRemove()
        if self.name is not None:
            self.prefix = self.name
        self.actions = zope.component.getMultiAdapter(
            (self, self.request, self), z3c.form.interfaces.IActions)
        self.actions.update()

    @btn.buttonAndHandler(IMultiFormButtons['add'],
        condition=attrgetter('allowAdding'))
    def handleAdd(self, action):
        self.appendAddingWidget()

    @btn.buttonAndHandler(IMultiFormButtons['remove'],
        condition=attrgetter('allowRemoving'))
    def handleRemove(self, action):
        self.removeWidgets([widget.name for widget in self.widgets
                            if ('%s.remove' % (widget.name)) in self.request])

    def update(self):
        """See z3c.form.interfaces.IWidget."""
        super(MultiWidget, self).update()
        self.updateActions()
        self.actions.execute()
        # Update again, as conditions may change
        self.updateActions()
        try:
            # force to disable sub widget labels
            for widget in self.key_widgets:
                if widget is not None:
                    widget.showLabel = self.showKeyWidgetLabel
            # show label
            for widget in self.widgets:
                widget.showLabel = self.showWidgetLabel
        except AttributeError:
            pass


@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getMultiFieldWidget(field, request):
    """IFieldWidget factory for MultiWidget."""
    widget = MultiWidget(request)
    return z3c.form.widget.FieldWidget(field, widget)


@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getMultiFieldWidgetDispatcher(field, value_type, request):
    """IFieldWidget factory for MultiWidget."""
    return getMultiFieldWidget(field, request)



##############################################################################
#
# Copyright (c) 2007 Projekt01 GmbH.
# All Rights Reserved.
#
##############################################################################
"""
$Id: select2.py 3942 2014-03-24 08:59:21Z roger.ineichen $
"""
__docformat__ = 'restructuredtext'

import zope.interface

import z3c.form.widget
import z3c.form.interfaces

import j01.select2.widget

from j01.form import interfaces
from j01.form.widget.widget import WidgetMixin


TAG_INIT_SELECTION_SCRIPT = j01.select2.widget.TAG_INIT_SELECTION_SCRIPT
RESULT_QUERY_SCRIPT = j01.select2.widget.RESULT_QUERY_SCRIPT
SINGLE_TAG_INIT_SELECTION_SCRIPT = \
    j01.select2.widget.SINGLE_TAG_INIT_SELECTION_SCRIPT
LIVELIST_INIT_SELECTION_SCRIPT = \
    j01.select2.widget.LIVELIST_INIT_SELECTION_SCRIPT
LIVELIST_RESULT_QUERY_SCRIPT = j01.select2.widget.LIVELIST_RESULT_QUERY_SCRIPT


class Select2Widget(WidgetMixin, j01.select2.widget.Select2Widget):
    """Selects input widget"""

    zope.interface.implementsOnly(interfaces.ISelect2Widget)

    klass = u'select2-control'
    css = u'select2'


class TagListSelect2Widget(WidgetMixin,
    j01.select2.widget.TagListSelect2Widget):
    """Widget for IList of ITextLine
    
    This widget is based on a IList of ITextLine field, this means we can enter
    custom text data and the JSON-RPC callback is used for autosuggest
    useable input.

    """

    zope.interface.implementsOnly(interfaces.ITagListSelect2Widget)

    klass = u'select2-taglist-control'
    css = u'select2-taglist'


class SingleTagSelect2Widget(WidgetMixin,
    j01.select2.widget.SingleTagSelect2Widget):
    """Widget for ITextLine supporting jsonrpc autosuggest callback"""

    zope.interface.implementsOnly(interfaces.ISingleTagSelect2Widget)

    klass = u'select2-singletag-control'
    css = u'select2-singletag'


class LiveListSelect2Widget(WidgetMixin,
    j01.select2.widget.LiveListSelect2Widget):
    """Widget for IList of IChoice"""

    zope.interface.implementsOnly(interfaces.ILiveListSelect2Widget)

    klass = u'select2-livelist-control'
    css = u'select2-livelist'


# HTML select element
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getSelect2Widget(field, request):
    """IFieldWidget factory for Select2Widget."""
    return z3c.form.widget.FieldWidget(field, Select2Widget(request))


@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getSingleSelect2Widget(field, request):
    """IFieldWidget factory for Select2Widget."""
    widget = z3c.form.widget.FieldWidget(field, Select2Widget(request))
    widget.multiple = None
    return widget


# tagging
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getTagListSelect2Widget(field, request):
    """IFieldWidget factory for TagListSelect2Widget."""
    return z3c.form.widget.FieldWidget(field, TagListSelect2Widget(request))


# live list
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getLiveListSelect2Widget(field, request):
    """IFieldWidget factory for LiveListSelect2Widget."""
    return z3c.form.widget.FieldWidget(field, LiveListSelect2Widget(request))




###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Interfaces

$Id: interfaces.py 4891 2018-06-05 12:32:20Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.schema
import zope.interface
import zope.i18nmessageid

import z3c.form.interfaces

import j01.dialog.interfaces
import j01.jsonrpc.interfaces


###############################################################################
#
# form

class IForm(zope.interface.Interface):
    """Simple form"""

    refreshWidgets = zope.schema.Bool(
        title=u'Refresh widgets',
        description=(u'A flag, when set, causes form widgets to be updated '
                     u'again after action execution.'),
        default=False,
        required=True)

    refreshActions = zope.schema.Bool(
        title=u'Refresh actions',
        description=(u'A flag, when set, causes form actions to be updated '
                     u'again after action execution.'),
        default=False,
        required=True)

    def setUpWidgetValidation(name):
        """Support for single widget ssetup used by j01.validate"""


class IDisplayForm(IForm, z3c.form.interfaces.IDisplayForm):
    """Display Form"""


class IAddForm(IForm, z3c.form.interfaces.IAddForm):
    """Add form."""


class IEditForm(IForm, z3c.form.interfaces.IEditForm):
    """Edit form."""


###############################################################################
#
# jsonrpc form

class IJSONRPCForm(IForm, j01.jsonrpc.interfaces.IJSONRPCForm):
    """JSON-RPC base form mixin class."""


class IJSONRPCAddForm(IJSONRPCForm, j01.jsonrpc.interfaces.IJSONRPCAddForm):
    """JSON-RPC based add form."""


class IJSONRPCEditForm(IJSONRPCForm, j01.jsonrpc.interfaces.IJSONRPCEditForm):
    """JSON-RPC based edit form."""


###############################################################################
#
# dialog form

class IDialogForm(IForm, j01.dialog.interfaces.IDialogPage):
    """Dialog form."""


class IDialogAddForm(IDialogForm, j01.dialog.interfaces.IDialogAddForm):
    """Dialog add form."""


class IDialogEditForm(IDialogForm, j01.dialog.interfaces.IDialogEditForm):
    """Dialog edit form."""


class IDialogDeleteForm(IDialogForm, j01.dialog.interfaces.IDialogDeleteForm):
    """Dialog delete form."""


class IDialogConfirmForm(IDialogForm, j01.dialog.interfaces.IDialogConfirmForm):
    """Dialog confirm form."""


###############################################################################
#
# widgets

class IWidget(z3c.form.interfaces.IWidget):
    """Enhanced widget supporting addons and description text"""

    description = zope.schema.TextLine(
        title=u'Description',
        description=u'Description',
        default=None,
        required=False)

    addOnBefore = zope.schema.TextLine(
        title=u'Addon before widget',
        description=u'Addon before widget',
        default=None,
        required=False)

    addOnAfter = zope.schema.TextLine(
        title=u'Addon after widget',
        description=u'Addon after widget',
        default=None,
        required=False)

    addOnWrapper = zope.schema.TextLine(
        title=u'Addon widget wrapper',
        description=u'Addon widget wrapper',
        default=None,
        required=False)

    # layout element rendering conditions
    showLabel = zope.schema.Bool(
        title=u'Show widget label',
        description=u'Show widget label',
        default=True,
        required=False)

    showDescription = zope.schema.Bool(
        title=u'Show widget description',
        description=u'Show widget description',
        default=True,
        required=False)

    showRequired = zope.schema.Bool(
        title=u'Show widget required',
        description=u'Show widget required',
        default=True,
        required=False)

    showError = zope.schema.Bool(
        title=u'Show widget error',
        description=u'Show widget error',
        default=True,
        required=False)


# html5 text
class ITextWidget(IWidget):
    """Text widget with placeholder and hint support"""

    pattern = zope.schema.ASCIILine(
        title=u'Validation pattern',
        description=u'Validation pattern',
        default=None,
        required=False)

    placeholder = zope.schema.TextLine(
        title=u'Placeholder',
        description=u'Placeholder',
        default=None,
        required=False)


class IEMailWidget(ITextWidget):
    """EMail input type widget"""


class IDateWidget(ITextWidget):
    """Date input type widget"""


class IDatetimeWidget(ITextWidget):
    """Datetime input type widget"""


class IDatetimeLocalWidget(ITextWidget):
    """Datetime local input type widget"""


class ITimeWidget(ITextWidget):
    """Time input type widget"""


class IWeekWidget(ITextWidget):
    """Week input type widget"""


class IMonthWidget(ITextWidget):
    """Month input type widget"""


class IColorWidget(ITextWidget):
    """Color input type widget"""


class ISearchWidget(ITextWidget):
    """Search input type widget"""


class IURLWidget(ITextWidget):
    """Search input type widget"""


class INumberWidget(ITextWidget):
    """Number input type widget"""


class ITelWidget(ITextWidget):
    """Tel input type widget"""


# checkbox
class ICheckBoxWidget(IWidget, z3c.form.interfaces.ICheckBoxWidget):
    """CheckBoxWidget using a div wrapper for option tags"""


class ICheckBoxInlineWidget(ICheckBoxWidget):
    """CheckBoxWidget using a span wrapper for option tags"""


class ISingleCheckBoxWidget(ICheckBoxWidget,
    z3c.form.interfaces.ISingleCheckBoxWidget):
    """Single checkbox using a div wrapper for option tags"""


class ISingleCheckBoxInlineWidget(ISingleCheckBoxWidget):
    """SingleCheckBoxWidget using a span wrapper for option tags"""


class ICheckBoxPickerWidget(ICheckBoxWidget):
    """Checkbox picker using a div wrapper for option tags"""


class ICheckBoxInlinePickerWidget(ICheckBoxWidget):
    """Checkbox picker using a span wrapper for option tags"""


class ISingleCheckBoxPickerWidget(ISingleCheckBoxWidget):
    """Single checkbox using a div wrapper for option tags"""


class ISingleCheckBoxInlinePickerWidget(ISingleCheckBoxPickerWidget):
    """SingleCheckBoxWidget using a span wrapper for option tags"""


# file
class IFileWidget(IWidget, z3c.form.interfaces.IFileWidget):
    """File widget."""


# multi
class IMultiWidget(IWidget, z3c.form.interfaces.IMultiWidget):
    """None Term based sequence widget base.

    The multi widget is used for ITuple, IList or IDict if no other widget is defined.

    Some IList or ITuple are using another specialized widget if they can
    choose from a collection. e.g. a IList of IChoice. The base class of such
    widget is the ISequenceWidget.

    This widget can handle none collection based sequences and offers add or
    remove values to or from the sequence. Each sequence value get rendered by
    it's own relevant widget. e.g. IList of ITextLine or ITuple of IInt
    """


# password
class IPasswordWidget(ITextWidget, z3c.form.interfaces.IPasswordWidget):
    """Password widget with placeholder and hint support"""


class IPasswordConfirmationWidget(ITextWidget,
    z3c.form.interfaces.IPasswordWidget):
    """Password including confirmation field widget."""


# radio
class IRadioWidget(IWidget, z3c.form.interfaces.IRadioWidget):
    """Radio widget."""

    showNoValueItem = zope.schema.Bool(
        title=u'Show None as value in selection',
        description=u'Show None as value in selection',
        default=False,
        required=True)

class IRadioInlineWidget(IRadioWidget):
    """Radio inline widget."""

class IRadioPickerWidget(IRadioWidget):
    """Radio picker widget."""

class IRadioInlinePickerWidget(IRadioWidget):
    """Radio inline picker widget."""


# select
class ISelectWidget(IWidget, z3c.form.interfaces.ISelectWidget):
    """Select widget with ITerms option."""


class IMultiSelectWidget(ISelectWidget):
    """Multi select widget"""


class ISelectPickerWidget(ISelectWidget):
    """MultiSelectPickerWidget"""


class IMultiSelectPickerWidget(ISelectPickerWidget, IMultiSelectWidget):
    """MultiSelectPickerWidget"""


class IGroupSelectWidget(ISelectWidget):
    """Select widget with optgroup support"""


# text lines
class ITextLinesWidget(IWidget, z3c.form.interfaces.ITextLinesWidget):
    """Text lines widget."""


# textarea
class ITextAreaWidget(IWidget, z3c.form.interfaces.ITextAreaWidget):
    """Text widget."""


# dictionary
class IDictKeyValueWidget(IWidget, z3c.form.interfaces.ITextAreaWidget):
    """Dict with key:value values as textarea widget."""


# proxy dictionary (<proto>:<host:port>)
class IProxyWidget(IDictKeyValueWidget):
    """Dict with <key>:<host:port> values as textarea widget.

    The key:value get split on first ':' as separator. Any following ':' will
    get joined as value.
    """


###############################################################################
#
# only availabe if j01.datepicker is available

try:
    import j01.datepicker.interfaces
    class IDatePickerWidget(IWidget,
        j01.datepicker.interfaces.IDatePickerWidget):
        """DatePicker date widget."""
except ImportError:
    pass


###############################################################################
#
# only availabe if j01.datetimepicker is available

try:
    import j01.datetimepicker.interfaces
    class IDateTimePickerWidget(IWidget,
        j01.datetimepicker.interfaces.IDateTimePickerWidget):
        """DatePicker date widget."""
except ImportError:
    pass


###############################################################################
#
# only availabe if j01.rater is available

try:
    import j01.rater.interfaces

    class IRatingWidget(IWidget,
        j01.rater.interfaces.IRatingWidget):
        """IRatingWidget widget"""

    class IFiveStarRatingWidget(IRatingWidget,
        j01.rater.interfaces.IFiveStarRatingWidget):
        """IFiveStarRatingWidget widget"""

    class IFiveHalfStarRatingWidget(IRatingWidget,
        j01.rater.interfaces.IFiveHalfStarRatingWidget):
        """IFiveHalfStarRatingWidget widget"""

    class IFiveHalfStarFullRatingWidget(IRatingWidget,
        j01.rater.interfaces.IFiveHalfStarFullRatingWidget):
        """IFiveHalfStarFullRatingWidget widget"""

except ImportError:
    pass


###############################################################################
#
# only availabe if j01.select2 is available

try:
    import j01.select2.interfaces
    # select
    class ISelect2Widget(IWidget, j01.select2.interfaces.ISelect2Widget):
        """Select2 widget for ISequence of IChoice."""


    # tagging
    class ITagListSelect2Widget(IWidget,
        j01.select2.interfaces.ITagListSelect2Widget):
        """Select2 widget for IList of ITextLine"""


    # single tag
    class ISingleTagSelect2Widget(IWidget,
        j01.select2.interfaces.ISingleTagSelect2Widget):
        """Select2 widget for ITextLine"""


    # livelist
    class ILiveListSelect2Widget(IWidget,
        j01.select2.interfaces.ILiveListSelect2Widget):
        """Select2 widget for IList of IChoice offering live autosuggest"""
except ImportError:
    pass


###############################################################################
#
# only availabe if j01.selectordered is available

try:
    import j01.selectordered.interfaces
    # select ordered
    class IOrderedSelectWidget(IWidget,
        j01.selectordered.interfaces.IOrderedSelectWidget):
        """Select ordered widget"""
except ImportError:
    pass


###############################################################################
#
# only availabe if m01.gmap is available

try:
    import m01.gmap.interfaces
    class IGMapWidget(IWidget, m01.gmap.interfaces.IGMapWidget):
        """GMap widget"""

    class IGeoPointGMapWidget(IWidget, m01.gmap.interfaces.IGeoPointGMapWidget):
        """GeoPoint GMap widget"""
except ImportError:
    pass



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Textlines widget
$Id: textlines.py 3934 2014-03-17 07:38:52Z roger.ineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface

import zope.component
import zope.interface
import zope.schema.interfaces

import z3c.form.interfaces
import z3c.form.widget
import z3c.form.browser.textlines

from j01.form import interfaces
from j01.form.layer import IFormLayer
from j01.form.widget.widget import WidgetMixin


class TextLinesWidget(WidgetMixin, z3c.form.browser.textlines.TextLinesWidget):
    """Textarea widget implementation."""

    zope.interface.implementsOnly(interfaces.ITextLinesWidget)

    klass = u'textlines form-control'
    css = u'textlines'
    value = u''


# get
@zope.component.adapter(zope.schema.interfaces.IField, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getTextLinesWidget(field, request):
    """IFieldWidget factory for TextWidget."""
    return z3c.form.widget.FieldWidget(field, TextLinesWidget(request))



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Password widget
$Id: password.py 4640 2017-09-14 15:47:35Z roger.ineichen $
"""
__docformat__ = 'restructuredtext'

# use html.escape for python3
import cgi

import zope.interface
import zope.component
import zope.i18n
import zope.i18nmessageid
import zope.schema
import zope.schema.interfaces

import z3c.form.interfaces
import z3c.form.converter
import z3c.form.validator
import z3c.form.widget

import j01.form.exceptions
from j01.form import interfaces
from j01.form.layer import IFormLayer
from j01.form.widget.text import TextWidget

_ = zope.i18nmessageid.MessageFactory('p01')


class PasswordWidget(TextWidget):
    """Input type password widget implementation."""

    zope.interface.implementsOnly(interfaces.IPasswordWidget)

    klass = u'password-control form-control'
    css = u'password'


LABEL_PASSWORD_CONFIRMATION = """
<div class="labelPasswordConfirmation">
  <label for="%s">
    <span>%s</span>
  </label>
</div>
"""

class PasswordConfirmationWidget(TextWidget):
    """Input type password widget implementation."""
    zope.interface.implementsOnly(interfaces.IPasswordConfirmationWidget)

    klass = u'password-confirmation-control form-control'
    css = u'password-confirmation'
    _type = 'password'

    @property
    def labelPasswordConfirmation(self):
        label = zope.i18n.translate(_(u'Password confirmation'),
            context=self.request)
        for_ = '%s-confirm' % self.id
        # escaped label because we use structure vew/labelPasswordConfirmation
        return LABEL_PASSWORD_CONFIRMATION % (for_, cgi.escape(label))


class PasswordRequiredValue(object):
    """Knows if input is required or not."""

    zope.interface.implements(z3c.form.interfaces.IValue)
    zope.component.adapts(zope.interface.Interface, zope.interface.Interface,
        zope.interface.Interface, zope.schema.interfaces.IPassword,
        interfaces.IPasswordConfirmationWidget)

    def __init__(self, context, request, form, field, widget):
        self.context = context
        self.request = request
        self.form = form
        self.field = field
        self.widget = widget

    def get(self):
        """Returns the value for the required field."""
        if self.field.required == True and \
            self.widget.value != self.field.missing_value:
            # change the required flag at the field
            self.field.required = False
        return self.field.required


class PasswordConfirmationValidator(z3c.form.validator.SimpleFieldValidator):
    """Simple Field Validator"""
    zope.interface.implements(z3c.form.interfaces.IValidator)
    zope.component.adapts(
        zope.interface.Interface,
        zope.interface.Interface,
        zope.interface.Interface,
        zope.schema.interfaces.IPassword,
        interfaces.IPasswordConfirmationWidget)

    def validate(self, value):
        """See interfaces.IValidator"""
        # we get a value if the password is equal to the confirmation value or
        # if password and confirmation is empty, we get the existing value
        # stored in the field

        # don't validate emtpy value if the widget was set to required = False
        # by the PasswordRequiredValue adapter
        requestValue = self.request.get(self.widget.name, 1)
        confirmValue = self.request.get(self.widget.name + '.confirm', 2)
        if confirmValue == u'' and requestValue == u'' and \
            self.field.required == False:
            return

        # compare both field values with each others
        if requestValue != confirmValue:
            raise j01.form.exceptions.PasswordComparsionError

        # default validation if we not allready get done
        field = self.field
        if self.context is not None:
            field = field.bind(self.context)
        return field.validate(value)


# password confirmation converter
class PasswordConfirmationDataConverter(z3c.form.converter.FieldDataConverter):
    """A data converter using the field's ``fromUnicode()`` method."""
    zope.component.adapts(zope.schema.interfaces.IFromUnicode,
        interfaces.IPasswordConfirmationWidget)

    def toFieldValue(self, value):
        """See interfaces.IDataConverter"""
        # check for empty form input
        confirm = self.widget.request.get(self.widget.name + '.confirm', None)
        if value == u'' and confirm == u'' and self.field.required == False:
            # if there is an empty value, we return the field value if widget
            # was set to required = False by the PasswordRequiredValue adapter
            return self.field.query(self.widget.context)
        return self.field.fromUnicode(value)


# get
@zope.component.adapter(zope.schema.interfaces.IPassword, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getPasswordWidget(field, request):
    """IFieldWidget factory for IPasswordWidget."""
    return z3c.form.widget.FieldWidget(field, PasswordWidget(request))


@zope.component.adapter(zope.schema.interfaces.IPassword, IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getPasswordConfirmationWidget(field, request):
    """IFieldWidget factory for PasswordConfirmationWidget."""
    return z3c.form.widget.FieldWidget(field, PasswordConfirmationWidget(request))



###############################################################################
#
# Copyright (c) 2014 Projekt01 GmbH.
# All Rights Reserved.
#
###############################################################################
"""Text widget
$Id: text.py 3979 2014-03-25 10:59:26Z roger.ineichen $
"""
__docformat__ = 'restructuredtext'

import zope.component
import zope.interface
import zope.schema.interfaces

import z3c.form.interfaces
import z3c.form.widget
import z3c.form.browser.widget

from j01.form import interfaces
from j01.form.layer import IFormLayer
from j01.form.widget.widget import WidgetMixin


JAVASCRIPT = """
<script type="text/javascript">
  $("#%s").j01Placeholder();
</script>
"""


class TextWidget(WidgetMixin, z3c.form.browser.widget.HTMLTextInputWidget,
    z3c.form.widget.Widget):
    """Text input type widget"""

    zope.interface.implementsOnly(interfaces.ITextWidget)
    _type = 'text'

    klass = u'text-control form-control'
    css = u'text'
    value = u''

    addon = None
    placeholder = None
    pattern = None

    def extract(self, default=z3c.form.interfaces.NO_VALUE):
        """Return NO_VALUE if value is placeholder """
        value = self.request.get(self.name, default)
        if self.placeholder == value:
            # return empty string
            return u''
        else:
            # return default or NO_VALUE
            return value

    @property
    def javascript(self):
        if self.placeholder is not None:
            return JAVASCRIPT % self.id


class EMailWidget(TextWidget):
    """EMail input type widget"""

    zope.interface.implementsOnly(interfaces.IEMailWidget)
    _type = 'email'
    klass = u'email-control form-control'
    css = u'email'


class DateWidget(TextWidget):
    """Date input type widget"""

    zope.interface.implementsOnly(interfaces.IDateWidget)
    _type = 'date'
    klass = u'date-control form-control'
    css = u'date'


class DatetimeWidget(TextWidget):
    """Datetime input type widget"""

    zope.interface.implementsOnly(interfaces.IDatetimeWidget)
    _type = 'datetime'
    klass = u'datetime-control form-control'
    css = u'datetime'


class DatetimeLocalWidget(TextWidget):
    """Datetime local input type widget"""

    zope.interface.implementsOnly(interfaces.IDatetimeLocalWidget)
    _type = 'datetime-local'
    klass = u'datetime-local-control form-control'
    css = u'datetime-local'


class TimeWidget(TextWidget):
    """Time input type widget"""

    zope.interface.implementsOnly(interfaces.ITimeWidget)
    _type = 'time'
    klass = u'time-control form-control'
    css = u'time'


class WeekWidget(TextWidget):
    """Week input type widget"""

    zope.interface.implementsOnly(interfaces.IWeekWidget)
    _type = 'week'
    klass = u'week-control form-control'
    css = u'week'


class MonthWidget(TextWidget):
    """Month input type widget"""

    zope.interface.implementsOnly(interfaces.IMonthWidget)
    _type = 'month'
    klass = u'month-control form-control'
    css = u'month'


class ColorWidget(TextWidget):
    """Color input type widget"""

    zope.interface.implementsOnly(interfaces.IColorWidget)
    _type = 'color'
    klass = u'color-control form-control'
    css = u'color'


class SearchWidget(TextWidget):
    """Search input type widget"""

    zope.interface.implementsOnly(interfaces.ISearchWidget)
    _type = 'search'
    klass = u'search-control form-control'
    css = u'search'


class URLWidget(TextWidget):
    """URL input type widget"""

    zope.interface.implementsOnly(interfaces.IURLWidget)
    _type = 'url'
    klass = u'url-control form-control'
    css = u'url'


class NumberWidget(TextWidget):
    """Number input type widget"""

    zope.interface.implementsOnly(interfaces.INumberWidget)
    _type = 'number'
    klass = u'number-control form-control'
    css = u'number'


class TelWidget(TextWidget):
    """Tel input type widget"""

    zope.interface.implementsOnly(interfaces.ITelWidget)
    _type = 'tel'
    klass = u'tel-control form-control'
    css = u'tel'


# text
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getTextWidget(field, request):
    """IFieldWidget factory for TextWidget."""
    return z3c.form.widget.FieldWidget(field, TextWidget(request))


# email
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getEMailWidget(field, request):
    """IFieldWidget factory for EMailWidget."""
    return z3c.form.widget.FieldWidget(field, EMailWidget(request))


# date
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getDateWidget(field, request):
    """IFieldWidget factory for DateWidget."""
    return z3c.form.widget.FieldWidget(field, DateWidget(request))


# datetime
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getDatetimeWidget(field, request):
    """IFieldWidget factory for DateWidget."""
    return z3c.form.widget.FieldWidget(field, DatetimeWidget(request))


# datetime-local
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getDatetimeLocalWidget(field, request):
    """IFieldWidget factory for DatetimeLocalWidget."""
    return z3c.form.widget.FieldWidget(field, DatetimeLocalWidget(request))


# time
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getTimeWidget(field, request):
    """IFieldWidget factory for TimeWidget."""
    return z3c.form.widget.FieldWidget(field, TimeWidget(request))


# week
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getWeekWidget(field, request):
    """IFieldWidget factory for WeekWidget."""
    return z3c.form.widget.FieldWidget(field, WeekWidget(request))


# month
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getMonthWidget(field, request):
    """IFieldWidget factory for MonthWidget."""
    return z3c.form.widget.FieldWidget(field, MonthWidget(request))


# color
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getColorWidget(field, request):
    """IFieldWidget factory for ColorWidget."""
    return z3c.form.widget.FieldWidget(field, ColorWidget(request))


# search
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getSearchWidget(field, request):
    """IFieldWidget factory for SearchWidget."""
    return z3c.form.widget.FieldWidget(field, SearchWidget(request))


# url
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getURLWidget(field, request):
    """IFieldWidget factory for URLWidget."""
    return z3c.form.widget.FieldWidget(field, URLWidget(request))


# number
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getNumberWidget(field, request):
    """IFieldWidget factory for NumberWidget."""
    return z3c.form.widget.FieldWidget(field, NumberWidget(request))


# tel
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def getTelWidget(field, request):
    """IFieldWidget factory for TelWidget."""
    return z3c.form.widget.FieldWidget(field, TelWidget(request))
